Class: Doing::WWID
Overview
Main "What Was I Doing" methods
Constant Summary
Constants included from Color
Color::ATTRIBUTES, Color::ATTRIBUTE_NAMES, Color::COLORED_REGEXP
Instance Attribute Summary collapse
-
#additional_configs ⇒ Object
readonly
Returns the value of attribute additional_configs.
-
#auto_tag ⇒ Object
Returns the value of attribute auto_tag.
-
#config ⇒ Object
Returns the value of attribute config.
-
#config_file ⇒ Object
Returns the value of attribute config_file.
-
#content ⇒ Object
readonly
Returns the value of attribute content.
-
#current_section ⇒ Object
readonly
Returns the value of attribute current_section.
-
#default_option ⇒ Object
Returns the value of attribute default_option.
-
#doing_file ⇒ Object
readonly
Returns the value of attribute doing_file.
Instance Method Summary collapse
-
#act_on(items, opt = {}) ⇒ Object
Perform actions on a set of entries.
-
#add_item(title, section = nil, opt = {}) ⇒ Object
Adds an entry.
- #all_tags(items, opt: {}) ⇒ Object
-
#archive(section = , options = {}) ⇒ Object
Move entries from a section to Archive or other specified section.
-
#autotag(text) ⇒ Object
Uses 'autotag' configuration to turn keywords into tags for time tracking.
-
#choose_section ⇒ String
Generate a menu of sections and allow user selection.
-
#choose_view ⇒ String
Generate a menu of views and allow user selection.
-
#create(filename = nil) ⇒ Object
Create a new doing file.
-
#dedup(items, no_overlap: false) ⇒ Object
Remove items from a list that already exist in @content.
-
#edit_last(section: 'All', options: {}) ⇒ Object
Edit the last entry.
-
#filter_items(items = Items.new, opt: {}) ⇒ Object
Filter items based on search criteria.
-
#fork_editor(input = '', message: :default) ⇒ Object
Create a process for an editor and wait for the file handle to return.
-
#format_input(input) ⇒ Array
Takes a multi-line string and formats it as an entry.
-
#format_time(seconds, human: false) ⇒ Object
Format human readable time from seconds.
- #fuzzy_filter_items(items, opt: {}) ⇒ Object
-
#get_interval(item, formatted: true, record: true) ⇒ Object
Gets the interval between entry's start date and @done date.
-
#get_view(title) ⇒ Object
Gets a view from configuration.
-
#guess_section(frag, guessed: false, suggest: false) ⇒ Object
Attempt to match a string with an existing section.
-
#guess_view(frag, guessed: false, suggest: false) ⇒ Object
Attempt to match a string with an existing view.
-
#import(paths, opt = {}) ⇒ Object
Imports external entries.
-
#init_doing_file(path = nil) ⇒ Object
Initializes the doing file.
-
#initialize ⇒ WWID
constructor
Initializes the object.
-
#interactive(opt = {}) ⇒ Object
Display an interactive menu of entries.
-
#last(times: true, section: nil, options: {}) ⇒ Object
Show the last entry.
-
#last_entry(opt = {}) ⇒ Object
Get the last entry.
-
#last_note(section = 'All') ⇒ Object
Return the content of the last note for a given section.
-
#list_date(dates, section, times = nil, output = nil, opt = {}) ⇒ Object
Display entries within a date range.
-
#list_section(opt = {}) ⇒ Object
Display contents of a section based on options.
-
#logger ⇒ Object
Logger.
-
#next_item(item, options = {}) ⇒ Item
Get next item in the index.
-
#recent(count = 10, section = nil, opt = {}) ⇒ Object
Show recent entries.
-
#repeat_item(item, opt = {}) ⇒ Object
Duplicate an item and add it as a new item.
-
#repeat_last(opt = {}) ⇒ Object
Restart the last entry.
-
#reset_item(item, resume: false) ⇒ Object
Reset start date to current time, optionally remove done tag (resume).
-
#restore_backup(file) ⇒ Object
Restore a backed up version of a file.
-
#rotate(opt = {}) ⇒ Object
Rename doing file with date and start fresh one.
-
#sections ⇒ Array
List sections.
-
#stop_start(target_tag, opt = {}) ⇒ Object
Accepts one tag and the raw text of a new item if the passed tag is on any item, it's replaced with @done.
- #tag_groups(items, opt: {}) ⇒ Object
-
#tag_last(opt = {}) ⇒ Object
Tag the last entry or X entries.
-
#tag_times(format: :text, sort_by_name: false, sort_order: 'asc') ⇒ Object
Get total elapsed time for all tags in selection.
-
#today(times = true, output = nil, opt = {}) ⇒ Object
Show all entries from the current day.
-
#views ⇒ Array
List available views.
-
#write(file = nil, backup: true) ⇒ Object
Write content to file or STDOUT.
-
#yesterday(section, times = nil, output = nil, opt = {}) ⇒ Object
Show entries from the previous day.
Methods included from Color
attributes, coloring?, #support?, #uncolor
Constructor Details
#initialize ⇒ WWID
Initializes the object.
25 26 27 28 29 30 |
# File 'lib/doing/wwid.rb', line 25 def initialize @timers = {} @recorded_items = [] @content = Items.new @auto_tag = true end |
Instance Attribute Details
#additional_configs ⇒ Object (readonly)
Returns the value of attribute additional_configs.
15 16 17 |
# File 'lib/doing/wwid.rb', line 15 def additional_configs @additional_configs end |
#auto_tag ⇒ Object
Returns the value of attribute auto_tag.
17 18 19 |
# File 'lib/doing/wwid.rb', line 17 def auto_tag @auto_tag end |
#config ⇒ Object
Returns the value of attribute config.
17 18 19 |
# File 'lib/doing/wwid.rb', line 17 def config @config end |
#config_file ⇒ Object
Returns the value of attribute config_file.
17 18 19 |
# File 'lib/doing/wwid.rb', line 17 def config_file @config_file end |
#content ⇒ Object (readonly)
Returns the value of attribute content.
15 16 17 |
# File 'lib/doing/wwid.rb', line 15 def content @content end |
#current_section ⇒ Object (readonly)
Returns the value of attribute current_section.
15 16 17 |
# File 'lib/doing/wwid.rb', line 15 def current_section @current_section end |
#default_option ⇒ Object
Returns the value of attribute default_option.
17 18 19 |
# File 'lib/doing/wwid.rb', line 17 def default_option @default_option end |
#doing_file ⇒ Object (readonly)
Returns the value of attribute doing_file.
15 16 17 |
# File 'lib/doing/wwid.rb', line 15 def doing_file @doing_file end |
Instance Method Details
#act_on(items, opt = {}) ⇒ Object
Perform actions on a set of entries. If no valid action is included in the opt hash and the terminal is a TTY, a menu will be presented
816 817 818 819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 860 861 862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968 969 970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 1051 1052 1053 1054 1055 |
# File 'lib/doing/wwid.rb', line 816 def act_on(items, opt = {}) actions = i[editor delete tag flag finish cancel archive output save_to again resume] has_action = false single = items.count == 1 actions.each do |a| if opt[a] has_action = true break end end unless has_action actions = [ 'add tag', 'remove tag', 'cancel', 'delete', 'finish', 'flag', 'archive', 'move', 'edit', 'output formatted' ] actions.concat(['resume/repeat', 'begin/reset']) if items.count == 1 choice = Prompt.choose_from(actions, prompt: 'What do you want to do with the selected items? > ', multiple: true, sorted: false, fzf_args: ["--height=#{actions.count + 3}", '--tac', '--no-sort', '--info=hidden']) return unless choice to_do = choice.strip.split(/\n/) to_do.each do |action| case action when /resume/ opt[:resume] = true when /reset/ opt[:reset] = true when /(add|remove) tag/ type = action =~ /^add/ ? 'add' : 'remove' raise InvalidArgument, "'add tag' and 'remove tag' can not be used together" if opt[:tag] print "#{yellow("Tag to #{type}: ")}#{reset}" tag = $stdin.gets next if tag =~ /^ *$/ opt[:tag] = tag.strip.sub(/^@/, '') opt[:remove] = true if type == 'remove' when /output formatted/ plugins = Plugins.available_plugins(type: :export).sort output_format = Prompt.choose_from(plugins, prompt: 'Which output format? > ', fzf_args: [ "--height=#{plugins.count + 3}", '--tac', '--no-sort', '--info=hidden' ]) next if tag =~ /^ *$/ raise UserCancelled unless output_format opt[:output] = output_format.strip res = opt[:force] ? false : Prompt.yn('Save to file?', default_response: 'n') if res print "#{yellow('File path/name: ')}#{reset}" filename = $stdin.gets.strip next if filename.empty? opt[:save_to] = filename end when /archive/ opt[:archive] = true when /delete/ opt[:delete] = true when /edit/ opt[:editor] = true when /finish/ opt[:finish] = true when /cancel/ opt[:cancel] = true when /move/ section = choose_section.strip opt[:move] = section.strip unless section =~ /^ *$/ when /flag/ opt[:flag] = true end end end if opt[:resume] || opt[:reset] raise InvalidArgument, 'resume and restart can only be used on a single entry' if items.count > 1 item = items[0] if opt[:resume] && !opt[:reset] repeat_item(item, { editor: opt[:editor] }) elsif opt[:reset] res = if item.('done', :and) && !opt[:resume] opt[:force] ? true : Prompt.yn('Remove @done tag?', default_response: 'y') else opt[:resume] end @content.update_item(item, reset_item(item, resume: res)) end write(@doing_file) return end if opt[:delete] res = opt[:force] ? true : Prompt.yn("Delete #{items.size} items?", default_response: 'y') if res items.each { |i| @content.delete_item(i, single: items.count == 1) } write(@doing_file) end return end if opt[:flag] tag = @config['marker_tag'] || 'flagged' items.map! do |i| i.tag(tag, date: false, remove: opt[:remove], single: single) end end if opt[:finish] || opt[:cancel] tag = 'done' items.map! do |i| if i.should_finish? should_date = !opt[:cancel] && i.should_time? i.tag(tag, date: should_date, remove: opt[:remove], single: single) end end end if opt[:tag] tag = opt[:tag] items.map! do |i| i.tag(tag, date: false, remove: opt[:remove], single: single) end end if opt[:archive] || opt[:move] section = opt[:archive] ? 'Archive' : guess_section(opt[:move]) items.map! { |i| i.move_to(section, label: true) } end write(@doing_file) if opt[:editor] editable_items = [] items.each do |i| editable = "#{i.date.strftime('%F %R')} | #{i.title}" old_note = i.note ? i.note.strip_lines.join("\n") : nil editable += "\n#{old_note}" unless old_note.nil? editable_items << editable end divider = "\n-----------\n" notice =" # - You may delete entries, but leave all divider lines (---) in place.\n # - Start and @done dates replaced with a time string (yesterday 3pm) will\n # be parsed automatically. Do not delete the pipe (|) between start date\n # and entry title.\n EONOTICE\n input = \"\#{editable_items.map(&:strip).join(divider)}\\n\\n\#{notice}\"\n\n new_items = fork_editor(input).split(/\#{divider}/)\n\n new_items.each_with_index do |new_item, i|\n input_lines = new_item.split(/[\\n\\r]+/).delete_if(&:ignore?)\n first_line = input_lines[0]&.strip\n\n if first_line.nil? || first_line =~ /^\#{divider.strip}$/ || first_line.strip.empty?\n @content.delete_item(items[i], single: new_items.count == 1)\n Doing.logger.count(:deleted)\n else\n date, title, note = format_input(new_item)\n\n note.map!(&:strip)\n note.delete_if(&:ignore?)\n item = items[i]\n old_item = item.dup\n item.date = date || items[i].date\n item.title = title\n item.note = note\n if (item.equal?(old_item))\n Doing.logger.count(:skipped, level: :debug)\n else\n Doing.logger.count(:updated)\n end\n end\n end\n\n write(@doing_file)\n end\n\n return unless opt[:output]\n\n items.map! do |i|\n i.title = \"\#{i.title} @project(\#{i.section})\"\n i\n end\n\n @content = Items.new\n @content.concat(items)\n @content.add_section(Section.new('Export'), log: false)\n options = { section: 'Export' }\n\n if opt[:output] =~ /doing/\n options[:output] = 'template'\n options[:template] = '- %date | %title%note'\n else\n options[:output] = opt[:output]\n options[:template] = opt[:template] || nil\n end\n\n output = list_section(options)\n\n if opt[:save_to]\n file = File.expand_path(opt[:save_to])\n if File.exist?(file)\n # Create a backup copy for the undo command\n FileUtils.cp(file, \"\#{file}~\")\n end\n\n File.open(file, 'w+') do |f|\n f.puts output\n end\n\n logger.warn('File written:', file)\n else\n Doing::Pager.page output\n end\nend\n" |
#add_item(title, section = nil, opt = {}) ⇒ Object
Adds an entry
323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 341 342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 |
# File 'lib/doing/wwid.rb', line 323 def add_item(title, section = nil, opt = {}) section ||= @config['current_section'] @content.add_section(section, log: false) opt[:back] ||= opt[:date] ? opt[:date] : Time.now opt[:date] ||= Time.now note = Note.new opt[:timed] ||= false note.add(opt[:note]) if opt[:note] title = [title.strip.cap_first] title = title.join(' ') if @auto_tag title = autotag(title) title.(@config['default_tags']) unless @config['default_tags'].empty? end title.compress! entry = Item.new(opt[:back], title.strip, section) entry.note = note items = @content.dup if opt[:timed] items.reverse! items.each_with_index do |i, x| next if i.title =~ / @done/ items[x].title = "#{i.title} @done(#{opt[:back].strftime('%F %R')})" break end end @content.push(entry) # logger.count(:added, level: :debug) logger.info('New entry:', %(added "#{entry.title}" to #{section})) end |
#all_tags(items, opt: {}) ⇒ Object
531 532 533 534 535 |
# File 'lib/doing/wwid.rb', line 531 def (items, opt: {}) = [] items.each { |item| .concat(item.).uniq! } .sort end |
#archive(section = , options = {}) ⇒ Object
Move entries from a section to Archive or other specified section
1499 1500 1501 1502 1503 1504 1505 1506 1507 1508 1509 1510 1511 1512 1513 1514 1515 1516 1517 1518 1519 1520 |
# File 'lib/doing/wwid.rb', line 1499 def archive(section = @config['current_section'], = {}) count = [:keep] || 0 destination = [:destination] || 'Archive' = [:tags] || [] bool = [:bool] || :and section = choose_section if section.nil? || section =~ /choose/i archive_all = section =~ /^all$/i # && !(tags.nil? || tags.empty?) section = guess_section(section) unless archive_all @content.add_section(destination, log: true) # add_section(Section.new('Archive')) if destination =~ /^archive$/i && [email protected]?('Archive') destination = guess_section(destination) if @content.section?(destination) && (@content.section?(section) || archive_all) do_archive(section, destination, { count: count, tags: , bool: bool, search: [:search], label: [:label], before: [:before] }) write(doing_file) else raise InvalidArgument, 'Either source or destination does not exist' end end |
#autotag(text) ⇒ Object
Uses 'autotag' configuration to turn keywords into tags for time tracking. Does not repeat tags in a title, and only converts the first instance of an untagged keyword
1718 1719 1720 1721 1722 1723 1724 1725 1726 1727 1728 1729 1730 1731 1732 1733 1734 1735 1736 1737 1738 1739 1740 1741 1742 1743 1744 1745 1746 1747 1748 1749 1750 1751 1752 1753 1754 1755 1756 1757 1758 1759 1760 1761 1762 1763 1764 1765 1766 1767 1768 1769 1770 1771 1772 1773 1774 1775 1776 1777 1778 1779 1780 1781 1782 1783 1784 1785 1786 1787 1788 1789 1790 1791 1792 1793 1794 1795 1796 1797 1798 1799 1800 1801 1802 1803 1804 1805 1806 1807 |
# File 'lib/doing/wwid.rb', line 1718 def autotag(text) return unless text return text unless @auto_tag original = text.dup = text.scan(/@\w+/).map { |t| t.sub(/^@/, '') } tagged = { whitelisted: [], synonyms: [], transformed: [], replaced: [] } @config['autotag']['whitelist'].each do |tag| next if text =~ /@#{tag}\b/i text.sub!(/(?<= |\A)(#{tag.strip})(?= |\Z)/i) do |m| m.downcase! unless tag =~ /[A-Z]/ tagged[:whitelisted].push(m) "@#{m}" end end @config['autotag']['synonyms'].each do |tag, v| v.each do |word| next unless text =~ /\b#{word}\b/i unless .include?(tag) || tagged[:whitelisted].include?(tag) tagged[:synonyms].push(tag) tagged[:synonyms] = tagged[:synonyms].uniq end end end if @config['autotag'].key? 'transform' @config['autotag']['transform'].each do |tag| next unless tag =~ /\S+:\S+/ rx, r = tag.split(/:/) flag_rx = %r{/([r]+)$} if r =~ flag_rx flags = r.match(flag_rx)[1].split(//) r.sub!(flag_rx, '') end r.gsub!(/\$/, '\\') rx.sub!(/^@?/, '@') regex = Regexp.new("(?<= |\\A)#{rx}(?= |\\Z)") text.sub!(regex) do m = Regexp.last_match new_tag = r m.to_a.slice(1, m.length - 1).each_with_index do |v, idx| new_tag.gsub!("\\#{idx + 1}", v) end # Replace original tag if /r if flags&.include?('r') tagged[:replaced].concat(new_tag.split(/ /).map { |t| t.sub(/^@/, '') }) new_tag.split(/ /).map { |t| t.sub(/^@?/, '@') }.join(' ') else tagged[:transformed].concat(new_tag.split(/ /).map { |t| t.sub(/^@/, '') }) tagged[:transformed] = tagged[:transformed].uniq m[0] end end end end logger.debug('Autotag:', "whitelisted tags: #{tagged[:whitelisted].log_tags}") unless tagged[:whitelisted].empty? logger.debug('Autotag:', "synonyms: #{tagged[:synonyms].log_tags}") unless tagged[:synonyms].empty? logger.debug('Autotag:', "transforms: #{tagged[:transformed].log_tags}") unless tagged[:transformed].empty? logger.debug('Autotag:', "transform replaced: #{tagged[:replaced].log_tags}") unless tagged[:replaced].empty? = tagged[:synonyms].concat(tagged[:transformed]) .sort! .uniq! text.() unless .empty? if text == original logger.debug('Autotag:', "no change to \"#{text.strip}\"") else = tagged[:whitelisted].concat().concat(tagged[:replaced]) logger.debug('Autotag:', "added #{new_tags.log_tags} to \"#{text.strip}\"") logger.count(:autotag, level: :info, count: 1, message: 'autotag updated %count %items') end text. end |
#choose_section ⇒ String
Generate a menu of sections and allow user selection
1390 1391 1392 1393 |
# File 'lib/doing/wwid.rb', line 1390 def choose_section choice = Prompt.choose_from(@content.section_titles.sort, prompt: 'Choose a section > ', fzf_args: ['--height=60%']) choice ? choice.strip : choice end |
#choose_view ⇒ String
Generate a menu of views and allow user selection
1409 1410 1411 1412 |
# File 'lib/doing/wwid.rb', line 1409 def choose_view choice = Prompt.choose_from(views.sort, prompt: 'Choose a view > ', fzf_args: ['--height=60%']) choice ? choice.strip : choice end |
#create(filename = nil) ⇒ Object
Create a new doing file
113 114 115 116 117 118 119 120 |
# File 'lib/doing/wwid.rb', line 113 def create(filename = nil) filename = @doing_file if filename.nil? return if File.exist?(filename) && File.stat(filename).size.positive? File.open(filename, 'w+') do |f| f.puts "#{@config['current_section']}:" end end |
#dedup(items, no_overlap: false) ⇒ Object
Remove items from a list that already exist in @content
367 368 369 370 371 372 373 374 375 376 377 378 |
# File 'lib/doing/wwid.rb', line 367 def dedup(items, no_overlap: false) items.delete_if do |item| duped = false @content.each do |comp| duped = no_overlap ? item.overlapping_time?(comp) : item.same_time?(comp) break if duped end logger.count(:skipped, level: :debug, message: '%count overlapping %items') if duped # logger.log_now(:debug, 'Skipped:', "overlapping entry: #{item.title}") if duped duped end end |
#edit_last(section: 'All', options: {}) ⇒ Object
Edit the last entry
1203 1204 1205 1206 1207 1208 1209 1210 1211 1212 1213 1214 1215 1216 1217 1218 1219 1220 1221 1222 1223 1224 1225 1226 1227 1228 1229 1230 1231 |
# File 'lib/doing/wwid.rb', line 1203 def edit_last(section: 'All', options: {}) [:section] = guess_section(section) item = last_entry() if item.nil? logger.debug('Skipped:', 'No entries found') return end content = ["#{item.date.strftime('%F %R')} | #{item.title.dup}"] content << item.note.strip_lines.join("\n") unless item.note.empty? new_item = fork_editor(content.join("\n")) date, title, note = format_input(new_item) date ||= item.date if title.nil? || title.empty? logger.debug('Skipped:', 'No content provided') elsif title == item.title && note.equal?(item.note) && date.equal?(item.date) logger.debug('Skipped:', 'No change in content') else item.date = date unless date.nil? item.title = title item.note.add(note, replace: true) logger.info('Edited:', item.title) write(@doing_file) end end |
#filter_items(items = Items.new, opt: {}) ⇒ Object
Filter items based on search criteria
598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 613 614 615 616 617 618 619 620 621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 759 |
# File 'lib/doing/wwid.rb', line 598 def filter_items(items = Items.new, opt: {}) time_rx = /^(\d{1,2}+(:\d{1,2}+)?( *(am|pm))?|midnight|noon)$/ if items.nil? || items.empty? section = opt[:section] ? guess_section(opt[:section]) : 'All' items = section =~ /^all$/i ? @content.dup : @content.in_section(section) end opt[:time_filter] = [nil, nil] if opt[:from] && !opt[:date_filter] date_string = opt[:from] case date_string when / (to|through|thru|(un)?til|-+) / dates = date_string.split(/ (?:to|through|thru|(?:un)?til|-+) /) if dates[0].strip =~ time_rx && dates[-1].strip =~ time_rx time_start = dates[0].strip time_end = dates[-1].strip else start = dates[0].chronify(guess: :begin) finish = dates[-1].chronify(guess: :end) end when time_rx time_start = date_string time_end = nil else start = date_string.chronify(guess: :begin) finish = false end if time_start opt[:time_filter] = [time_start, time_end] Doing.logger.debug('Parser:', "--from string interpreted as time span, from #{time_start ? time_start : '12am'} to #{time_end ? time_end : '11:59pm'}") else raise InvalidTimeExpression, 'Unrecognized date string' unless start opt[:date_filter] = [start, finish] Doing.logger.debug('Parser:', "--from string interpreted as #{start.strftime('%F %R')} -- #{finish ? finish.strftime('%F %R') : 'now'}") end end if opt[:before] =~ time_rx opt[:time_filter][1] = opt[:before] opt[:before] = nil end if opt[:after] =~ time_rx opt[:time_filter][0] = opt[:after] opt[:after] = nil end items.sort_by! { |item| [item.date, item.title.downcase] }.reverse filtered_items = items.select do |item| keep = true if opt[:unfinished] finished = item.('done', :and) finished = opt[:not] ? !finished : finished keep = false if finished end if keep && opt[:tag] opt[:tag_bool] ||= :and tag_match = opt[:tag].nil? || opt[:tag].empty? ? true : item.(opt[:tag], opt[:tag_bool]) keep = false unless tag_match keep = opt[:not] ? !keep : keep end if keep && opt[:search] search_match = if opt[:search].nil? || opt[:search].empty? true else item.search(opt[:search], case_type: opt[:case].normalize_case, fuzzy: opt[:fuzzy]) end keep = false unless search_match keep = opt[:not] ? !keep : keep end if keep && opt[:date_filter]&.length == 2 start_date = opt[:date_filter][0] end_date = opt[:date_filter][1] in_date_range = if end_date item.date >= start_date && item.date <= end_date else item.date.strftime('%F') == start_date.strftime('%F') end keep = false unless in_date_range keep = opt[:not] ? !keep : keep end if keep && opt[:time_filter][0] || opt[:time_filter][1] start_string = if opt[:time_filter][0].nil? "#{item.date.strftime('%Y-%m-%d')} 12am" else "#{item.date.strftime('%Y-%m-%d')} #{opt[:time_filter][0]}" end start_time = start_string.chronify(guess: :begin) end_string = if opt[:time_filter][1].nil? "#{item.date.next_day.strftime('%Y-%m-%d')} 12am" else "#{item.date.strftime('%Y-%m-%d')} #{opt[:time_filter][1]}" end end_time = end_string.chronify(guess: :end) in_time_range = item.date >= start_time && item.date <= end_time keep = false unless in_time_range keep = opt[:not] ? !keep : keep end keep = false if keep && opt[:only_timed] && !item.interval if keep && opt[:tag_filter] && !opt[:tag_filter]['tags'].empty? keep = item.(opt[:tag_filter]['tags'], opt[:tag_filter]['bool']) keep = opt[:not] ? !keep : keep end if keep && opt[:before] time_string = opt[:before] if time_string =~ time_rx cutoff = "#{item.date.strftime('%Y-%m-%d')} #{time_string}".chronify(guess: :begin) else cutoff = time_string.chronify(guess: :begin) end keep = cutoff && item.date <= cutoff keep = opt[:not] ? !keep : keep end if keep && opt[:after] time_string = opt[:after] if time_string =~ time_rx cutoff = "#{item.date.strftime('%Y-%m-%d')} #{time_string}".chronify(guess: :end) else cutoff = time_string.chronify(guess: :end) end keep = cutoff && item.date >= cutoff keep = opt[:not] ? !keep : keep end if keep && opt[:today] keep = item.date >= Date.today.to_time && item.date < Date.today.next_day.to_time keep = opt[:not] ? !keep : keep elsif keep && opt[:yesterday] keep = item.date >= Date.today.prev_day.to_time && item.date < Date.today.to_time keep = opt[:not] ? !keep : keep end keep end count = opt[:count]&.positive? ? opt[:count] : filtered_items.length output = Items.new if opt[:age] =~ /^o/i output.concat(filtered_items.slice(0, count).reverse) else output.concat(filtered_items.reverse.slice(0, count)) end output end |
#fork_editor(input = '', message: :default) ⇒ Object
Create a process for an editor and wait for the file handle to return
127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 |
# File 'lib/doing/wwid.rb', line 127 def fork_editor(input = '', message: :default) # raise NonInteractive, 'Non-interactive terminal' unless $stdout.isatty || ENV['DOING_EDITOR_TEST'] raise MissingEditor, 'No EDITOR variable defined in environment' if Util.default_editor.nil? tmpfile = Tempfile.new(['doing', '.md']) File.open(tmpfile.path, 'w+') do |f| f.puts input unless .nil? f.puts == :default ? "# The first line is the entry title, any lines after that are added as a note" : end end pid = Process.fork { system("#{Util.editor_with_args} #{tmpfile.path}") } trap('INT') do begin Process.kill(9, pid) rescue StandardError Errno::ESRCH end tmpfile.unlink tmpfile.close! exit 0 end Process.wait(pid) begin if $?.exitstatus == 0 input = IO.read(tmpfile.path) else exit_now! 'Cancelled' end ensure tmpfile.close tmpfile.unlink end input.split(/\n/).delete_if(&:ignore?).join("\n") end |
#format_input(input) ⇒ Array
Takes a multi-line string and formats it as an entry
177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 |
# File 'lib/doing/wwid.rb', line 177 def format_input(input) raise EmptyInput, 'No content in entry' if input.nil? || input.strip.empty? input_lines = input.split(/[\n\r]+/).delete_if(&:ignore?) title = input_lines[0]&.strip raise EmptyInput, 'No content in first line' if title.nil? || title.strip.empty? date = nil iso_rx = /\d{4}-\d\d-\d\d \d\d:\d\d/ done_rx = /(?<=^| )@(?<tag>done|finished|completed?)\((?<date>.*?)\)/i date_rx = /^(?:\s*- )?(?<date>.*?) \| (?=\S)/ title.gsub!(done_rx) do m = Regexp.last_match t = m['tag'] d = m['date'] parsed_date = d =~ date_rx ? Time.parse(d) : d.chronify(guess: :begin) parsed_date.nil? ? m[0] : "@#{t}(#{parsed_date.strftime('%F %R')})" end if title =~ date_rx m = title.match(date_rx) d = m['date'] date = if d =~ iso_rx Time.parse(d) else d.chronify(guess: :begin) end title.sub!(date_rx, '').strip! end note = Note.new note.add(input_lines[1..-1]) if input_lines.length > 1 # If title line ends in a parenthetical, use that as the note if note.empty? && title =~ /\s+\(.*?\)$/ title.sub!(/\s+\((?<note>.*?)\)$/) do m = Regexp.last_match note.add(m['note']) '' end end note.strip_lines! note.compress [date, title, note] end |
#format_time(seconds, human: false) ⇒ Object
Format human readable time from seconds
1976 1977 1978 1979 1980 1981 1982 1983 1984 1985 1986 1987 1988 1989 1990 1991 1992 1993 1994 1995 1996 |
# File 'lib/doing/wwid.rb', line 1976 def format_time(seconds, human: false) return [0, 0, 0] if seconds.nil? if seconds.instance_of?(String) && seconds =~ /(\d+):(\d+):(\d+)/ h = Regexp.last_match(1) m = Regexp.last_match(2) s = Regexp.last_match(3) seconds = (h.to_i * 60 * 60) + (m.to_i * 60) + s.to_i end minutes = (seconds / 60).to_i hours = (minutes / 60).to_i if human minutes = (minutes % 60).to_i [0, hours, minutes] else days = (hours / 24).to_i hours = (hours % 24).to_i minutes = (minutes % 60).to_i [days, hours, minutes] end end |
#fuzzy_filter_items(items, opt: {}) ⇒ Object
549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 |
# File 'lib/doing/wwid.rb', line 549 def fuzzy_filter_items(items, opt: {}) scannable = items.map.with_index { |item, idx| "#{item.title} #{item.note.join(' ')}".gsub(/[|*?!]/, '') + "|#{idx}" }.join("\n") fzf_args = [ '--multi', %(--filter="#{opt[:search].sub(/^'?/, "'")}"), '--no-sort', '-d "\|"', '--nth=1' ] if opt[:case] fzf_args << case opt[:case].normalize_case when :sensitive '+i' when :ignore '-i' end end # fzf_args << '-e' if opt[:exact] # puts fzf_args.join(' ') res = `echo #{Shellwords.escape(scannable)}|#{Prompt.fzf} #{fzf_args.join(' ')}` selected = Items.new res.split(/\n/).each do |item| idx = item.match(/\|(\d+)$/)[1].to_i selected.push(items[idx]) end selected end |
#get_interval(item, formatted: true, record: true) ⇒ Object
Gets the interval between entry's start date and @done date
1959 1960 1961 1962 1963 1964 1965 1966 1967 1968 1969 |
# File 'lib/doing/wwid.rb', line 1959 def get_interval(item, formatted: true, record: true) if item.interval seconds = item.interval record_tag_times(item, seconds) if record return seconds.positive? ? seconds : false unless formatted return seconds.positive? ? format('%02d:%02d:%02d', *format_time(seconds)) : false end false end |
#get_view(title) ⇒ Object
Gets a view from configuration
1419 1420 1421 1422 1423 |
# File 'lib/doing/wwid.rb', line 1419 def get_view(title) return @config['views'][title] if @config['views'].has_key?(title) false end |
#guess_section(frag, guessed: false, suggest: false) ⇒ Object
Attempt to match a string with an existing section
240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 |
# File 'lib/doing/wwid.rb', line 240 def guess_section(frag, guessed: false, suggest: false) return 'All' if frag =~ /^all$/i frag ||= @config['current_section'] return frag.cap_first if @content.section?(frag) section = nil re = frag.split('').join('.*?') sections.each do |sect| next unless sect =~ /#{re}/i logger.debug('Match:', %(Assuming "#{sect}" from "#{frag}")) section = sect break end return section if suggest unless section || guessed alt = guess_view(frag, guessed: true, suggest: true) if alt meant_view = Prompt.yn("#{boldwhite("Did you mean")} `#{yellow("doing view #{alt}")}#{boldwhite}`?", default_response: 'n') raise WrongCommand.new("run again with #{"doing view #{alt}".boldwhite}", topic: 'Try again:') if meant_view end res = Prompt.yn("#{boldwhite}Section #{frag.yellow}#{boldwhite} not found, create it", default_response: 'n') if res @content.add_section(frag.cap_first, log: true) write(@doing_file) return frag.cap_first end raise InvalidSection.new("unknown section #{frag.bold.white}", topic: 'Missing:') end section ? section.cap_first : guessed end |
#guess_view(frag, guessed: false, suggest: false) ⇒ Object
Attempt to match a string with an existing view
286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 |
# File 'lib/doing/wwid.rb', line 286 def guess_view(frag, guessed: false, suggest: false) views.each { |view| return view if frag.downcase == view.downcase } view = false re = frag.split('').join('.*?') views.each do |v| next unless v =~ /#{re}/i logger.debug('Match:', %(Assuming "#{v}" from "#{frag}")) view = v break end unless view || guessed alt = guess_section(frag, guessed: true, suggest: true) raise InvalidView.new(%(unknown view #{frag.bold.white}), topic: 'Missing:') unless alt meant_view = Prompt.yn("Did you mean `doing show #{alt}`?", default_response: 'n') raise WrongCommand.new("run again with #{"doing show #{alt}".yellow}", topic: 'Try again:') if meant_view raise InvalidView.new(%(unknown view #{alt.bold.white}), topic: 'Missing:') end view end |
#import(paths, opt = {}) ⇒ Object
Imports external entries
386 387 388 389 390 391 392 393 394 395 396 397 398 399 |
# File 'lib/doing/wwid.rb', line 386 def import(paths, opt = {}) Plugins.plugins[:import].each do |_, | next unless opt[:type] =~ /^(#{options[:trigger].normalize_trigger})$/i if paths.count.positive? paths.each do |path| [:class].import(self, path, options: opt) end else [:class].import(self, nil, options: opt) end break end end |
#init_doing_file(path = nil) ⇒ Object
Initializes the doing file.
50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 |
# File 'lib/doing/wwid.rb', line 50 def init_doing_file(path = nil) @doing_file = File.(@config['doing_file']) if path.nil? create(@doing_file) unless File.exist?(@doing_file) input = IO.read(@doing_file) input = input.force_encoding('utf-8') if input.respond_to? :force_encoding logger.debug('Read:', "read file #{@doing_file}") elsif File.exist?(File.(path)) && File.file?(File.(path)) && File.stat(File.(path)).size.positive? @doing_file = File.(path) input = IO.read(File.(path)) input = input.force_encoding('utf-8') if input.respond_to? :force_encoding logger.debug('Read:', "read file #{File.expand_path(path)}") elsif path.length < 256 @doing_file = File.(path) create(path) input = IO.read(File.(path)) input = input.force_encoding('utf-8') if input.respond_to? :force_encoding logger.debug('Read:', "read file #{File.expand_path(path)}") end @other_content_top = [] @other_content_bottom = [] section = nil lines = input.split(/[\n\r]/) lines.each do |line| next if line =~ /^\s*$/ if line =~ /^(\S[\S ]+):\s*(@\S+\s*)*$/ section = Regexp.last_match(1) @content.add_section(Section.new(section, original: line), log: false) elsif line =~ /^\s*- (\d{4}-\d\d-\d\d \d\d:\d\d) \| (.*)/ if section.nil? section = 'Uncategorized' @content.add_section(Section.new(section, original: 'Uncategorized:'), log: false) end date = Regexp.last_match(1).strip title = Regexp.last_match(2).strip item = Item.new(date, title, section) @content.push(item) elsif @content.count.zero? # if content[section].items.length - 1 == current @other_content_top.push(line) elsif line =~ /^\S/ @other_content_bottom.push(line) else prev_item = @content.last prev_item.note = Note.new unless prev_item.note prev_item.note.add(line) # end end end Hooks.trigger :post_read, self end |
#interactive(opt = {}) ⇒ Object
Display an interactive menu of entries
Options hash is shared with #filter_items and #act_on
768 769 770 771 772 773 774 775 776 777 778 779 780 781 782 783 784 785 786 787 788 789 790 791 792 793 |
# File 'lib/doing/wwid.rb', line 768 def interactive(opt = {}) opt[:section] = opt[:section] ? guess_section(opt[:section]) : 'All' search = nil if opt[:search] search = opt[:search] search.sub!(/^'?/, "'") if opt[:exact] opt[:search] = search end opt[:query] = opt[:search] if opt[:search] && !opt[:query] opt[:query] = "!#{opt[:query]}" if opt[:not] opt[:multiple] = true opt[:show_if_single] = true = i[after before case date_filter from fuzzy not search section].each_with_object({}) { |k, hsh| hsh[k] = opt[k] } items = filter_items(Items.new, opt: ) selection = Prompt.choose_from_items(items, include_section: opt[:section] =~ /^all$/i, **opt) raise NoResults, 'no items selected' if selection.nil? || selection.empty? act_on(selection, opt) end |
#last(times: true, section: nil, options: {}) ⇒ Object
Show the last entry
1672 1673 1674 1675 1676 1677 1678 1679 1680 1681 1682 1683 1684 1685 1686 1687 1688 1689 1690 1691 1692 1693 1694 1695 1696 1697 1698 1699 1700 1701 1702 1703 1704 1705 1706 1707 1708 1709 |
# File 'lib/doing/wwid.rb', line 1672 def last(times: true, section: nil, options: {}) section = section.nil? || section =~ /all/i ? 'All' : guess_section(section) cfg = @config['templates']['last'].deep_merge(@config['templates']['default']).deep_merge({ 'wrap_width' => @config['wrap_width'] || 0, 'date_format' => @config['default_date_format'], 'order' => @config['order'] || 'asc', 'tags_color' => @config['tags_color'], 'duration' => @config['duration'], 'interval_format' => @config['interval_format'] }) [:duration] ||= cfg['duration'] || false [:interval_format] ||= cfg['interval_format'] || 'text' opts = { section: section, wrap_width: cfg['wrap_width'], count: 1, format: cfg['date_format'], template: cfg['template'], times: times, duration: [:duration], interval_format: [:interval_format], case: [:case], not: [:negate], config_template: 'last' } if [:tag] opts[:tag_filter] = { 'tags' => [:tag], 'bool' => [:tag_bool] } end opts[:search] = [:search] if [:search] list_section(opts) end |
#last_entry(opt = {}) ⇒ Object
Get the last entry
507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 |
# File 'lib/doing/wwid.rb', line 507 def last_entry(opt = {}) opt[:tag_bool] ||= :and opt[:section] ||= @config['current_section'] items = filter_items(Items.new, opt: opt) logger.debug('Filtered:', "Parameters matched #{items.count} entries") if opt[:interactive] last_entry = Prompt.choose_from_items(items, include_section: opt[:section] =~ /^all$/i, menu: true, header: '', prompt: 'Select an entry > ', multiple: false, sort: false, show_if_single: true ) else last_entry = items.max_by { |item| item.date } end last_entry end |
#last_note(section = 'All') ⇒ Object
Return the content of the last note for a given section
407 408 409 410 411 412 413 414 415 416 417 418 |
# File 'lib/doing/wwid.rb', line 407 def last_note(section = 'All') section = guess_section(section) last_item = last_entry({ section: section }) raise NoEntryError, 'No entry found' unless last_item logger.log_now(:info, 'Edit note:', last_item.title) note = last_item.note&.to_s || '' "#{last_item.title}\n# EDIT BELOW THIS LINE ------------\n#{note}" end |
#list_date(dates, section, times = nil, output = nil, opt = {}) ⇒ Object
Display entries within a date range
1577 1578 1579 1580 1581 1582 1583 1584 1585 1586 1587 1588 1589 1590 1591 1592 1593 1594 1595 1596 |
# File 'lib/doing/wwid.rb', line 1577 def list_date(dates, section, times = nil, output = nil, opt = {}) opt[:totals] ||= false opt[:sort_tags] ||= false section = guess_section(section) # :date_filter expects an array with start and end date dates = [dates, dates] if dates.instance_of?(String) list_section({ section: section, count: 0, order: 'asc', date_filter: dates, times: times, output: output, totals: opt[:totals], duration: opt[:duration], sort_tags: opt[:sort_tags], config_template: 'default' }) end |
#list_section(opt = {}) ⇒ Object
Display contents of a section based on options
1430 1431 1432 1433 1434 1435 1436 1437 1438 1439 1440 1441 1442 1443 1444 1445 1446 1447 1448 1449 1450 1451 1452 1453 1454 1455 1456 1457 1458 1459 1460 1461 1462 1463 1464 1465 1466 1467 1468 1469 1470 1471 1472 1473 1474 1475 1476 1477 1478 1479 1480 1481 1482 1483 1484 1485 1486 1487 1488 1489 1490 |
# File 'lib/doing/wwid.rb', line 1430 def list_section(opt = {}) opt[:config_template] ||= 'default' cfg = @config.dig('templates', opt[:config_template]).deep_merge({ 'wrap_width' => @config['wrap_width'] || 0, 'date_format' => @config['default_date_format'], 'order' => @config['order'] || 'asc', 'tags_color' => @config['tags_color'], 'duration' => @config['duration'], 'interval_format' => @config['interval_format'] }) opt[:duration] ||= cfg['duration'] || false opt[:interval_format] ||= cfg['interval_format'] || 'text' opt[:count] ||= 0 opt[:age] ||= 'newest' opt[:format] ||= cfg['date_format'] opt[:order] ||= cfg['order'] || 'asc' opt[:tag_order] ||= 'asc' opt[:tags_color] = cfg['tags_color'] || false if opt[:tags_color].nil? opt[:template] ||= cfg['template'] # opt[:highlight] ||= true title = '' is_single = true if opt[:section].nil? opt[:section] = choose_section title = opt[:section] elsif opt[:section].instance_of?(String) title = if opt[:section] =~ /^all$/i if opt[:page_title] opt[:page_title] elsif opt[:tag_filter] && opt[:tag_filter]['bool'].normalize_bool != :not opt[:tag_filter]['tags'].map { |tag| "@#{tag}" }.join(' + ') else 'doing' end else guess_section(opt[:section]) end end items = filter_items(Items.new, opt: opt).reverse items.reverse! if opt[:order] =~ /^d/i if opt[:interactive] opt[:menu] = !opt[:force] opt[:query] = '' # opt[:search] opt[:multiple] = true selected = Prompt.choose_from_items(items, include_section: opt[:section] =~ /^all$/i, **opt) raise NoResults, 'no items selected' if selected.empty? act_on(selected, opt) return end opt[:output] ||= 'template' opt[:wrap_width] ||= @config['templates']['default']['wrap_width'] || 0 output(items, title, is_single, opt) end |
#logger ⇒ Object
Logger
Responds to :debug, :info, :warn, and :error
Each method takes a topic, and a message or block
Example: debug('Hooks', 'Hook 1 triggered')
41 42 43 |
# File 'lib/doing/wwid.rb', line 41 def logger @logger ||= Doing.logger end |
#next_item(item, options = {}) ⇒ Item
Get next item in the index
1190 1191 1192 1193 1194 1195 1196 |
# File 'lib/doing/wwid.rb', line 1190 def next_item(item, = {}) items = filter_items(Items.new, opt: ) idx = items.index(item) idx.positive? ? items[idx - 1] : nil end |
#recent(count = 10, section = nil, opt = {}) ⇒ Object
Show recent entries
1641 1642 1643 1644 1645 1646 1647 1648 1649 1650 1651 1652 1653 1654 1655 1656 1657 1658 1659 1660 1661 1662 1663 1664 |
# File 'lib/doing/wwid.rb', line 1641 def recent(count = 10, section = nil, opt = {}) times = opt[:t] || true opt[:totals] ||= false opt[:sort_tags] ||= false cfg = @config['templates']['recent'].deep_merge(@config['templates']['default']).deep_merge({ 'wrap_width' => @config['wrap_width'] || 0, 'date_format' => @config['default_date_format'], 'order' => @config['order'] || 'asc', 'tags_color' => @config['tags_color'], 'duration' => @config['duration'], 'interval_format' => @config['interval_format'] }) opt[:duration] ||= cfg['duration'] || false opt[:interval_format] ||= cfg['interval_format'] || 'text' section ||= @config['current_section'] section = guess_section(section) list_section({ section: section, wrap_width: cfg['wrap_width'], count: count, format: cfg['date_format'], template: cfg['template'], order: 'asc', times: times, totals: opt[:totals], sort_tags: opt[:sort_tags], tags_color: opt[:tags_color], config_template: 'recent' }) end |
#repeat_item(item, opt = {}) ⇒ Object
Duplicate an item and add it as a new item
445 446 447 448 449 450 451 452 453 454 455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 |
# File 'lib/doing/wwid.rb', line 445 def repeat_item(item, opt = {}) if item.should_finish? if item.should_time? item.title.tag!('done', value: Time.now.strftime('%F %R')) else item.title.tag!('done') end end # Remove @done tag title = item.title.sub(/\s*@done(\(.*?\))?/, '').chomp section = opt[:in].nil? ? item.section : guess_section(opt[:in]) @auto_tag = false note = opt[:note] || Note.new if opt[:editor] start = opt[:date] ? opt[:date] : Time.now to_edit = "#{start.strftime('%F %R')} | #{title}" to_edit += "\n#{note.strip_lines.join("\n")}" unless note.empty? new_item = fork_editor(to_edit) date, title, note = format_input(new_item) opt[:date] = date unless date.nil? if title.nil? || title.empty? logger.warn('Skipped:', 'No content provided') return end end # @content.update_item(original, item) add_item(title, section, { note: note, back: opt[:date], timed: true }) end |
#repeat_last(opt = {}) ⇒ Object
Restart the last entry
485 486 487 488 489 490 491 492 493 494 495 496 497 498 499 500 |
# File 'lib/doing/wwid.rb', line 485 def repeat_last(opt = {}) opt[:section] ||= 'all' opt[:section] = guess_section(opt[:section]) opt[:note] ||= [] opt[:tag] ||= [] opt[:tag_bool] ||= :and last = last_entry(opt) if last.nil? logger.warn('Skipped:', 'No previous entry found') return end repeat_item(last, opt) write(@doing_file) end |
#reset_item(item, resume: false) ⇒ Object
Reset start date to current time, optionally remove done tag (resume)
426 427 428 429 430 431 |
# File 'lib/doing/wwid.rb', line 426 def reset_item(item, resume: false) item.date = Time.now item.tag('done', remove: true) if resume logger.info('Reset:', %(Reset #{resume ? 'and resumed ' : ''} "#{item.title}" in #{item.section})) item end |
#restore_backup(file) ⇒ Object
Restore a backed up version of a file
1317 1318 1319 1320 1321 1322 1323 1324 |
# File 'lib/doing/wwid.rb', line 1317 def restore_backup(file) if File.exist?("#{file}~") FileUtils.cp("#{file}~", file) logger.warn('File update:', "Restored #{file.sub(/^#{Util.user_home}/, '~')}") else logger.error('Restore error:', 'No backup file found') end end |
#rotate(opt = {}) ⇒ Object
Rename doing file with date and start fresh one
1329 1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 1340 1341 1342 1343 1344 1345 1346 1347 1348 1349 1350 1351 1352 1353 1354 1355 1356 1357 1358 1359 1360 1361 1362 1363 1364 1365 1366 1367 1368 1369 1370 1371 1372 1373 1374 1375 1376 1377 1378 1379 1380 1381 1382 1383 |
# File 'lib/doing/wwid.rb', line 1329 def rotate(opt = {}) keep = opt[:keep] || 0 = [] .concat(opt[:tag].split(/ *, */).map { |t| t.sub(/^@/, '').strip }) if opt[:tag] bool = opt[:bool] || :and sect = opt[:section] !~ /^all$/i ? guess_section(opt[:section]) : 'all' section = guess_section(sect) section_items = @content.in_section(section) max = section_items.count - keep.to_i counter = 0 new_content = Items.new @content.each do |item| break if counter >= max if opt[:before] time_string = opt[:before] cutoff = time_string.chronify(guess: :begin) end unless ((!.empty? && !item.(, bool)) || (opt[:search] && !item.search(opt[:search].to_s)) || (opt[:before] && item.date >= cutoff)) new_item = @content.delete(item) raise DoingRuntimeError, "Error deleting item: #{item}" if new_item.nil? new_content.add_section(new_item.section, log: false) new_content.push(new_item) counter += 1 end end if counter.positive? logger.count(:rotated, level: :info, count: counter, message: "Rotated %count %items") else logger.info('Skipped:', 'No items were rotated') end write(@doing_file) file = @doing_file.sub(/(\.\w+)$/, "_#{Time.now.strftime('%Y-%m-%d')}\\1") if File.exist?(file) init_doing_file(file) @content.concat(new_content).uniq! logger.warn('File update:', "added entries to existing file: #{file}") else @content = new_content logger.warn('File update:', "created new file: #{file}") end write(file, backup: false) end |
#sections ⇒ Array
List sections
230 231 232 |
# File 'lib/doing/wwid.rb', line 230 def sections @content.section_titles end |
#stop_start(target_tag, opt = {}) ⇒ Object
Accepts one tag and the raw text of a new item if the passed tag is on any item, it's replaced with @done. if new_item is not nil, it's tagged with the passed tag and inserted. This is for use where only one instance of a given tag should exist (@meanwhile)
1248 1249 1250 1251 1252 1253 1254 1255 1256 1257 1258 1259 1260 1261 1262 1263 1264 1265 1266 1267 1268 1269 1270 1271 1272 1273 1274 1275 1276 1277 1278 1279 1280 1281 1282 1283 1284 1285 1286 1287 1288 1289 1290 1291 1292 1293 1294 |
# File 'lib/doing/wwid.rb', line 1248 def stop_start(target_tag, opt = {}) tag = target_tag.dup opt[:section] ||= @config['current_section'] opt[:archive] ||= false opt[:back] ||= Time.now opt[:new_item] ||= false opt[:note] ||= false opt[:section] = guess_section(opt[:section]) tag.sub!(/^@/, '') found_items = 0 @content.each_with_index do |item, i| next unless item.section == opt[:section] || opt[:section] =~ /all/i next unless item.title =~ /@#{tag}/ item.title.([tag, 'done'], remove: true) item.tag('done', value: opt[:back].strftime('%F %R')) found_items += 1 if opt[:archive] && opt[:section] != 'Archive' item.title = item.title.sub(/(?:@from\(.*?\))?(.*)$/, "\\1 @from(#{item.section})") item.move_to('Archive', label: false, log: false) logger.count(:completed_archived) logger.info('Completed/archived:', item.title) else logger.count(:completed) logger.info('Completed:', item.title) end end logger.debug('Skipped:', "No active @#{tag} tasks found.") if found_items.zero? if opt[:new_item] date, title, note = format_input(opt[:new_item]) opt[:back] = date unless date.nil? note.add(opt[:note]) if opt[:note] title.tag!(tag) add_item(title.cap_first, opt[:section], { note: note, back: opt[:back] }) end write(@doing_file) end |
#tag_groups(items, opt: {}) ⇒ Object
537 538 539 540 541 542 543 544 545 546 547 |
# File 'lib/doing/wwid.rb', line 537 def tag_groups(items, opt: {}) all_items = filter_items(items, opt: opt) = (all_items, opt: {}) tag_groups = {} .each do |tag| tag_groups[tag] ||= [] tag_groups[tag] = filter_items(all_items, opt: { tag: tag, tag_bool: :or }) end tag_groups end |
#tag_last(opt = {}) ⇒ Object
Tag the last entry or X entries
1066 1067 1068 1069 1070 1071 1072 1073 1074 1075 1076 1077 1078 1079 1080 1081 1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 1093 1094 1095 1096 1097 1098 1099 1100 1101 1102 1103 1104 1105 1106 1107 1108 1109 1110 1111 1112 1113 1114 1115 1116 1117 1118 1119 1120 1121 1122 1123 1124 1125 1126 1127 1128 1129 1130 1131 1132 1133 1134 1135 1136 1137 1138 1139 1140 1141 1142 1143 1144 1145 1146 1147 1148 1149 1150 1151 1152 1153 1154 1155 1156 1157 1158 1159 1160 1161 1162 1163 1164 1165 1166 1167 1168 1169 1170 1171 1172 1173 1174 1175 1176 1177 1178 1179 |
# File 'lib/doing/wwid.rb', line 1066 def tag_last(opt = {}) opt[:count] ||= 1 opt[:archive] ||= false opt[:tags] ||= ['done'] opt[:sequential] ||= false opt[:date] ||= false opt[:remove] ||= false opt[:autotag] ||= false opt[:back] ||= false opt[:unfinished] ||= false opt[:section] = opt[:section] ? guess_section(opt[:section]) : 'All' items = filter_items(Items.new, opt: opt) if opt[:interactive] items = Prompt.choose_from_items(items, include_section: opt[:section] =~ /^all$/i, menu: true, header: '', prompt: 'Select entries to tag > ', multiple: true, sort: true, show_if_single: true) raise NoResults, 'no items selected' if items.empty? end raise NoResults, 'no items matched your search' if items.empty? items.each do |item| added = [] removed = [] if opt[:autotag] new_title = autotag(item.title) if @auto_tag if new_title == item.title logger.count(:skipped, level: :debug, message: '%count unchaged %items') # logger.debug('Autotag:', 'No changes') else logger.count(:added_tags) logger.write(items.count == 1 ? :info : :debug, 'Tagged:', new_title) item.title = new_title end else if opt[:sequential] next_entry = next_item(item) done_date = if next_entry.nil? Time.now else next_entry.date - 60 end elsif opt[:took] if item.date + opt[:took] > Time.now item.date = Time.now - opt[:took] done_date = Time.now else done_date = item.date + opt[:took] end elsif opt[:back] done_date = if opt[:back].is_a? Integer item.date + opt[:back] else item.date + (opt[:back] - item.date) end else done_date = Time.now end opt[:tags].each do |tag| if tag == 'done' && !item.should_finish? Doing.logger.debug('Skipped:', "Item in never_finish: #{item.title}") logger.count(:skipped, level: :debug) next end tag = tag.strip if opt[:remove] || opt[:rename] rename_to = nil if opt[:rename] rename_to = tag tag = opt[:rename] end old_title = item.title.dup item.title.tag!(tag, remove: opt[:remove], rename_to: rename_to, regex: opt[:regex]) if old_title != item.title removed << tag added << rename_to if rename_to else logger.count(:skipped, level: :debug) end else old_title = item.title.dup should_date = opt[:date] && item.should_time? item.title.tag!('done', remove: true) if tag =~ /done/ && !should_date item.title.tag!(tag, value: should_date ? done_date.strftime('%F %R') : nil) added << tag if old_title != item.title end end end logger.log_change(tags_added: added, tags_removed: removed, item: item, single: items.count == 1) item.note.add(opt[:note]) if opt[:note] if opt[:archive] && opt[:section] != 'Archive' && (opt[:count]).positive? item.move_to('Archive', label: true) elsif opt[:archive] && opt[:count].zero? logger.warn('Skipped:', 'Archiving is skipped when operating on all entries') end end write(@doing_file) end |
#tag_times(format: :text, sort_by_name: false, sort_order: 'asc') ⇒ Object
Get total elapsed time for all tags in selection
1818 1819 1820 1821 1822 1823 1824 1825 1826 1827 1828 1829 1830 1831 1832 1833 1834 1835 1836 1837 1838 1839 1840 1841 1842 1843 1844 1845 1846 1847 1848 1849 1850 1851 1852 1853 1854 1855 1856 1857 1858 1859 1860 1861 1862 1863 1864 1865 1866 1867 1868 1869 1870 1871 1872 1873 1874 1875 1876 1877 1878 1879 1880 1881 1882 1883 1884 1885 1886 1887 1888 1889 1890 1891 1892 1893 1894 1895 1896 1897 1898 1899 1900 1901 1902 1903 1904 1905 1906 1907 1908 1909 1910 1911 1912 1913 1914 1915 1916 1917 1918 1919 1920 1921 1922 1923 1924 1925 1926 1927 1928 1929 1930 1931 1932 1933 1934 1935 1936 1937 1938 1939 1940 1941 1942 1943 |
# File 'lib/doing/wwid.rb', line 1818 def tag_times(format: :text, sort_by_name: false, sort_order: 'asc') return '' if @timers.empty? max = @timers.keys.sort_by { |k| k.length }.reverse[0].length + 1 total = @timers.delete('All') = @timers.delete_if { |_k, v| v == 0 } = if sort_by_name .sort_by { |k, _v| k } else .sort_by { |_k, v| v } end .reverse! if sort_order =~ /^asc/i case format when :html output = " <table>\n <caption id=\"tagtotals\">Tag Totals</caption>\n <colgroup>\n <col style=\"text-align:left;\"/>\n <col style=\"text-align:left;\"/>\n </colgroup>\n <thead>\n <tr>\n <th style=\"text-align:left;\">project</th>\n <th style=\"text-align:left;\">time</th>\n </tr>\n </thead>\n <tbody>\n" .reverse.each do |k, v| if v > 0 output += "<tr><td style='text-align:left;'>#{k}</td><td style='text-align:left;'>#{'%02d:%02d:%02d' % format_time(v)}</td></tr>\n" end end tail = " <tr>\n <td style=\"text-align:left;\" colspan=\"2\"></td>\n </tr>\n </tbody>\n <tfoot>\n <tr>\n <td style=\"text-align:left;\"><strong>Total</strong></td>\n <td style=\"text-align:left;\">\#{'%02d:%02d:%02d' % format_time(total)}</td>\n </tr>\n </tfoot>\n </table>\n" output + tail when :markdown pad = .map {|k, v| k }.group_by(&:size).max.last[0].length output = " | \#{' ' * (pad - 7) }project | time |\n | \#{'-' * (pad - 1)}: | :------- |\n EOS\n sorted_tags_data.reverse.each do |k, v|\n if v > 0\n output += \"| \#{' ' * (pad - k.length)}\#{k} | \#{'%02d:%02d:%02d' % format_time(v)} |\\n\"\n end\n end\n tail = \"[Tag Totals]\"\n output + tail\n when :json\n output = []\n sorted_tags_data.reverse.each do |k, v|\n d, h, m = format_time(v)\n output << {\n 'tag' => k,\n 'seconds' => v,\n 'formatted' => format('%<d>02d:%<h>02d:%<m>02d', d: d, h: h, m: m)\n }\n end\n output\n when :human\n output = []\n sorted_tags_data.reverse.each do |k, v|\n spacer = ''\n (max - k.length).times do\n spacer += ' '\n end\n _d, h, m = format_time(v, human: true)\n output.push(\"\u2503 \#{spacer}\#{k}:\#{format('%<h> 4dh %<m>02dm', h: h, m: m)} \u2503\")\n end\n\n header = '\u250F\u2501\u2501 Tag Totals '\n (max - 2).times { header += '\u2501' }\n header += '\u2513'\n footer = '\u2517'\n (max + 12).times { footer += '\u2501' }\n footer += '\u251B'\n divider = '\u2523'\n (max + 12).times { divider += '\u2501' }\n divider += '\u252B'\n output = output.empty? ? '' : \"\\n\#{header}\\n\#{output.join(\"\\n\")}\"\n d, h, m = format_time(total, human: true)\n output += \"\\n\#{divider}\"\n spacer = ''\n (max - 6).times do\n spacer += ' '\n end\n total = \"\u2503 \#{spacer}total: \"\n total += format('%<h> 4dh %<m>02dm', h: h, m: m)\n total += ' \u2503'\n output += \"\\n\#{total}\"\n output += \"\\n\#{footer}\"\n output\n else\n output = []\n sorted_tags_data.reverse.each do |k, v|\n spacer = ''\n (max - k.length).times do\n spacer += ' '\n end\n d, h, m = format_time(v)\n output.push(\"\#{k}:\#{spacer}\#{format('%<d>02d:%<h>02d:%<m>02d', d: d, h: h, m: m)}\")\n end\n\n output = output.empty? ? '' : \"\\n--- Tag Totals ---\\n\#{output.join(\"\\n\")}\"\n d, h, m = format_time(total)\n output += \"\\n\\nTotal tracked: \#{format('%<d>02d:%<h>02d:%<m>02d', d: d, h: h, m: m)}\\n\"\n output\n end\nend\n" |
#today(times = true, output = nil, opt = {}) ⇒ Object
Show all entries from the current day
1529 1530 1531 1532 1533 1534 1535 1536 1537 1538 1539 1540 1541 1542 1543 1544 1545 1546 1547 1548 1549 1550 1551 1552 1553 1554 1555 1556 1557 1558 1559 1560 1561 1562 1563 1564 1565 1566 |
# File 'lib/doing/wwid.rb', line 1529 def today(times = true, output = nil, opt = {}) opt[:totals] ||= false opt[:sort_tags] ||= false cfg = @config['templates']['today'].deep_merge(@config['templates']['default']).deep_merge({ 'wrap_width' => @config['wrap_width'] || 0, 'date_format' => @config['default_date_format'], 'order' => @config['order'] || 'asc', 'tags_color' => @config['tags_color'], 'duration' => @config['duration'], 'interval_format' => @config['interval_format'] }) opt[:duration] ||= cfg['duration'] || false opt[:interval_format] ||= cfg['interval_format'] || 'text' = { after: opt[:after], before: opt[:before], count: 0, duration: opt[:duration], from: opt[:from], format: cfg['date_format'], interval_format: opt[:interval_format], order: cfg['order'] || 'asc', output: output, section: opt[:section], sort_tags: opt[:sort_tags], template: cfg['template'], times: times, today: true, totals: opt[:totals], wrap_width: cfg['wrap_width'], tags_color: cfg['tags_color'], config_template: 'today' } list_section() end |
#views ⇒ Array
List available views
1400 1401 1402 |
# File 'lib/doing/wwid.rb', line 1400 def views @config.has_key?('views') ? @config['views'].keys : [] end |
#write(file = nil, backup: true) ⇒ Object
Write content to file or STDOUT
1301 1302 1303 1304 1305 1306 1307 1308 1309 1310 |
# File 'lib/doing/wwid.rb', line 1301 def write(file = nil, backup: true) Hooks.trigger :pre_write, self, file output = combined_content if file.nil? $stdout.puts output else Util.write_to_file(file, output, backup: backup) run_after if @config.key?('run_after') end end |
#yesterday(section, times = nil, output = nil, opt = {}) ⇒ Object
Show entries from the previous day
1606 1607 1608 1609 1610 1611 1612 1613 1614 1615 1616 1617 1618 1619 1620 1621 1622 1623 1624 1625 1626 1627 1628 1629 1630 1631 1632 |
# File 'lib/doing/wwid.rb', line 1606 def yesterday(section, times = nil, output = nil, opt = {}) opt[:totals] ||= false opt[:sort_tags] ||= false section = guess_section(section) y = (Time.now - (60 * 60 * 24)).strftime('%Y-%m-%d') opt[:after] = "#{y} #{opt[:after]}" if opt[:after] opt[:before] = "#{y} #{opt[:before]}" if opt[:before] = { after: opt[:after], before: opt[:before], count: 0, duration: opt[:duration], from: opt[:from], order: opt[:order], output: output, section: section, sort_tags: opt[:sort_tags], tag_order: opt[:tag_order], times: times, totals: opt[:totals], yesterday: true, config_template: 'today' } list_section() end |