Module: Doing::Prompt
- Extended by:
- Color
- Defined in:
- lib/doing/prompt.rb
Overview
Terminal Prompt methods
Constant Summary
Constants included from Color
Color::ATTRIBUTES, Color::ATTRIBUTE_NAMES, Color::COLORED_REGEXP
Class Attribute Summary collapse
Class Method Summary collapse
-
.choose_from(options, prompt: 'Make a selection: ', multiple: false, sorted: true, fzf_args: []) ⇒ String
Generate a menu of options and allow user selection.
-
.choose_from_items(items, **opt) ⇒ Object
Create an interactive menu to select from a set of Items.
- .fzf ⇒ Object
- .install_fzf ⇒ Object
-
.yn(question, default_response: false) ⇒ Bool
Ask a yes or no question in the terminal.
Methods included from Color
attributes, coloring?, support?, uncolor
Class Attribute Details
.default_answer ⇒ Object
15 16 17 |
# File 'lib/doing/prompt.rb', line 15 def default_answer @default_answer ||= false end |
.force_answer ⇒ Object
11 12 13 |
# File 'lib/doing/prompt.rb', line 11 def force_answer @force_answer ||= nil end |
Class Method Details
.choose_from(options, prompt: 'Make a selection: ', multiple: false, sorted: true, fzf_args: []) ⇒ String
Generate a menu of options and allow user selection
109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 |
# File 'lib/doing/prompt.rb', line 109 def choose_from(, prompt: 'Make a selection: ', multiple: false, sorted: true, fzf_args: []) return nil unless $stdout.isatty # fzf_args << '-1' # User is expecting a menu, and even if only one it seves as confirmation fzf_args << %(--prompt="#{prompt}") fzf_args << "--height=#{.count + 2}" fzf_args << '--info=inline' fzf_args << '--multi' if multiple header = "esc: cancel,#{multiple ? ' tab: multi-select, ctrl-a: select all,' : ''} return: confirm" fzf_args << %(--header="#{header}") .sort! if sorted res = `echo #{Shellwords.escape(.join("\n"))}|#{fzf} #{fzf_args.join(' ')}` return false if res.strip.size.zero? res end |
.choose_from_items(items, **opt) ⇒ Object
Create an interactive menu to select from a set of Items
142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 |
# File 'lib/doing/prompt.rb', line 142 def choose_from_items(items, **opt) return items unless $stdout.isatty return nil unless items.count.positive? case_sensitive = opt.fetch(:case, :smart).normalize_case header = opt.fetch(:header, 'Arrows: navigate, tab: mark for selection, ctrl-a: select all, enter: commit') prompt = opt.fetch(:prompt, 'Select entries to act on > ') query = opt.fetch(:query) { opt.fetch(:search, '') } include_section = opt.fetch(:include_section, false) pad = items.length.to_s.length = items.map.with_index do |item, i| out = [ format("%#{pad}d", i), ') ', format('%13s', item.date.relative_date), ' | ', item.title ] if include_section out.concat([ ' (', item.section, ') ' ]) end out.join('') end fzf_args = [ %(--header="#{header}"), %(--prompt="#{prompt.sub(/ *$/, ' ')}"), opt.fetch(:multiple) ? '--multi' : '--no-multi', '-0', '--bind ctrl-a:select-all', %(-q "#{query}"), '--info=inline' ] fzf_args.push('-1') unless opt.fetch(:show_if_single) fzf_args << case case_sensitive when :sensitive '+i' when :ignore '-i' end fzf_args << '-e' if opt.fetch(:exact, false) unless opt.fetch(:menu) raise InvalidArgument, "Can't skip menu when no query is provided" unless query && !query.empty? fzf_args.concat([%(--filter="#{query}"), opt.fetch(:sort) ? '' : '--no-sort']) end res = `echo #{Shellwords.escape(.join("\n"))}|#{fzf} #{fzf_args.join(' ')}` selected = [] res.split(/\n/).each do |item| idx = item.match(/^ *(\d+)\)/)[1].to_i selected.push(items[idx]) end opt.fetch(:multiple) ? selected : selected[0] end |
.fzf ⇒ Object
77 78 79 |
# File 'lib/doing/prompt.rb', line 77 def fzf @fzf ||= install_fzf end |
.install_fzf ⇒ Object
81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 |
# File 'lib/doing/prompt.rb', line 81 def install_fzf fzf_dir = File.join(File.dirname(__FILE__), '../helpers/fzf') FileUtils.mkdir_p(fzf_dir) unless File.directory?(fzf_dir) fzf_bin = File.join(fzf_dir, 'bin/fzf') return fzf_bin if File.exist?(fzf_bin) prev_level = Doing.logger.level Doing.logger.adjust_verbosity({ log_level: :info }) Doing.logger.log_now(:warn, 'Compiling and installing fzf -- this will only happen once') Doing.logger.log_now(:warn, 'fzf is copyright Junegunn Choi, MIT License <https://github.com/junegunn/fzf/blob/master/LICENSE>') system("'#{fzf_dir}/install' --bin --no-key-bindings --no-completion --no-update-rc --no-bash --no-zsh --no-fish &> /dev/null") unless File.exist?(fzf_bin) Doing.logger.log_now(:warn, 'Error installing, trying again as root') system("sudo '#{fzf_dir}/install' --bin --no-key-bindings --no-completion --no-update-rc --no-bash --no-zsh --no-fish &> /dev/null") end raise RuntimeError.new('Error installing fzf, please report at https://github.com/ttscoff/doing/issues') unless File.exist?(fzf_bin) Doing.logger.info("fzf installed to #{fzf}") Doing.logger.adjust_verbosity({ log_level: prev_level }) fzf_bin end |
.yn(question, default_response: false) ⇒ Bool
Ask a yes or no question in the terminal
29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 |
# File 'lib/doing/prompt.rb', line 29 def yn(question, default_response: false) unless @force_answer.nil? return @force_answer end default = if default_response.is_a?(String) default_response =~ /y/i ? true : false else default_response end # if global --default is set, answer default return default if @default_answer # if this isn't an interactive shell, answer default return default unless $stdout.isatty # clear the buffer if ARGV&.length ARGV.length.times do ARGV.shift end end system 'stty cbreak' cw = white cbw = boldwhite cbg = boldgreen cd = Color.default = unless default.nil? "#{cw}[#{default ? "#{cbg}Y#{cw}/#{cbw}n" : "#{cbw}y#{cw}/#{cbg}N"}#{cw}]#{cd}" else "#{cw}[#{cbw}y#{cw}/#{cbw}n#{cw}]#{cd}" end $stdout.syswrite "#{cbw}#{question.sub(/\?$/, '')} #{}#{cbw}?#{cd} " res = $stdin.sysread 1 puts system 'stty cooked' res.chomp! res.downcase! return default if res.empty? res =~ /y/i ? true : false end |