Class: WWID
- Inherits:
-
Object
- Object
- WWID
- Defined in:
- lib/doing/wwid.rb
Instance Attribute Summary collapse
-
#auto_tag ⇒ Object
Returns the value of attribute auto_tag.
-
#config ⇒ Object
Returns the value of attribute config.
-
#config_file ⇒ Object
Returns the value of attribute config_file.
-
#content ⇒ Object
Returns the value of attribute content.
-
#current_section ⇒ Object
Returns the value of attribute current_section.
-
#default_config_file ⇒ Object
Returns the value of attribute default_config_file.
-
#doing_file ⇒ Object
Returns the value of attribute doing_file.
-
#results ⇒ Object
Returns the value of attribute results.
-
#sections ⇒ Array
Section titles.
-
#user_home ⇒ Object
Returns the value of attribute user_home.
Instance Method Summary collapse
- #add_item(title, section = nil, opt = {}) ⇒ Object
- #add_section(title) ⇒ Object
- #archive(section = 'Currently', options = {}) ⇒ Object
-
#autotag(text) ⇒ Object
Does not repeat tags in a title, and only converts the first instance of an untagged keyword.
-
#choose_section ⇒ String
The selected section name.
-
#choose_view ⇒ String
The selected view name.
-
#chronify(input) ⇒ DateTime
Result.
-
#chronify_qty(qty) ⇒ Integer
Seconds.
-
#colors ⇒ String
ANSI escape sequence.
- #configure(opt = {}) ⇒ Object
- #create(filename = nil) ⇒ Object
-
#css_template ⇒ String
CSS template.
- #dedup(items, no_overlap = false) ⇒ Object
- #do_archive(sect, destination, opt = {}) ⇒ Object
- #edit_last(section: 'All', options: {}) ⇒ Object
-
#find_local_config ⇒ String
A file path.
- #fork_editor(input = '') ⇒ Object
-
#format_input(input) ⇒ Array
[(String)title, (Array)note].
- #get_view(title) ⇒ Object
- #guess_section(frag, guessed: false) ⇒ Object
- #guess_view(frag, guessed = false) ⇒ Object
-
#haml_template ⇒ String
HAML template.
- #import_timing(path, opt = {}) ⇒ Object
- #init_doing_file(path = nil) ⇒ Object
-
#initialize ⇒ WWID
constructor
A new instance of WWID.
- #last(times: true, section: nil, options: {}) ⇒ Object
- #last_entry(opt = {}) ⇒ Object
- #last_note(section = 'All') ⇒ Object
- #list_date(dates, section, times = nil, output = nil, opt = {}) ⇒ Object
- #list_section(opt = {}) ⇒ Object
- #note_last(section, note, replace: false) ⇒ Object
- #overlapping_time?(item_a, item_b) ⇒ Boolean
- #read_config(opt = {}) ⇒ Object
- #recent(count = 10, section = nil, opt = {}) ⇒ Object
- #restart_last(opt = {}) ⇒ Object
- #restore_backup(file) ⇒ Object
- #same_time?(item_a, item_b) ⇒ Boolean
- #stop_start(target_tag, opt = {}) ⇒ Object
- #tag_last(opt = {}) ⇒ Object
- #tag_times(format = 'text', sort_by_name = false) ⇒ Object
- #today(times = true, output = nil, opt = {}) ⇒ Object
- #update_item(old_item, new_item) ⇒ Object
-
#views ⇒ Array
View names.
- #write(file = nil) ⇒ Object
- #yesterday(section, times = nil, output = nil, opt = {}) ⇒ Object
-
#yn(question, default_response: false) ⇒ Bool
Yes or no.
Constructor Details
#initialize ⇒ WWID
Returns a new instance of WWID.
103 104 105 106 107 108 109 110 |
# File 'lib/doing/wwid.rb', line 103 def initialize @content = {} @doingrc_needs_update = false @default_config_file = '.doingrc' @interval_cache = {} @results = [] @auto_tag = true end |
Instance Attribute Details
#auto_tag ⇒ Object
Returns the value of attribute auto_tag.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def auto_tag @auto_tag end |
#config ⇒ Object
Returns the value of attribute config.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def config @config end |
#config_file ⇒ Object
Returns the value of attribute config_file.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def config_file @config_file end |
#content ⇒ Object
Returns the value of attribute content.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def content @content end |
#current_section ⇒ Object
Returns the value of attribute current_section.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def current_section @current_section end |
#default_config_file ⇒ Object
Returns the value of attribute default_config_file.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def default_config_file @default_config_file end |
#doing_file ⇒ Object
Returns the value of attribute doing_file.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def doing_file @doing_file end |
#results ⇒ Object
Returns the value of attribute results.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def results @results end |
#sections ⇒ Array
Returns section titles.
491 492 493 |
# File 'lib/doing/wwid.rb', line 491 def sections @sections end |
#user_home ⇒ Object
Returns the value of attribute user_home.
97 98 99 |
# File 'lib/doing/wwid.rb', line 97 def user_home @user_home end |
Instance Method Details
#add_item(title, section = nil, opt = {}) ⇒ Object
621 622 623 624 625 626 627 628 629 630 631 632 633 634 635 636 637 638 639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 |
# File 'lib/doing/wwid.rb', line 621 def add_item(title, section = nil, opt = {}) section ||= @current_section add_section(section) unless @content.has_key?(section) opt[:date] ||= Time.now opt[:note] ||= [] opt[:back] ||= Time.now opt[:timed] ||= false opt[:note] = [opt[:note]] if opt[:note].instance_of?(String) title = [title.strip.cap_first] title = title.join(' ') if @auto_tag title = autotag(title) unless @config['default_tags'].empty? = @config['default_tags'].map do |t| next if t.nil? dt = t.sub(/^ *@/, '').chomp if title =~ /@#{dt}/ '' else " @#{dt}" end end .delete_if { |t| t == '' } title += .join(' ') end end title.gsub!(/ +/, ' ') entry = { 'title' => title.strip, 'date' => opt[:back] } entry['note'] = opt[:note].map(&:chomp) unless opt[:note].join('').strip == '' items = @content[section]['items'] if opt[:timed] items.reverse! items.each_with_index do |i, x| next if i['title'] =~ / @done/ items[x]['title'] = "#{i['title']} @done(#{opt[:back].strftime('%F %R')})" break end items.reverse! end items.push(entry) @content[section]['items'] = items @results.push(%(Added "#{entry['title']}" to #{section})) end |
#add_section(title) ⇒ Object
500 501 502 503 |
# File 'lib/doing/wwid.rb', line 500 def add_section(title) @content[title.cap_first] = { 'original' => "#{title}:", 'items' => [] } @results.push(%(Added section "#{title.cap_first}")) end |
#archive(section = 'Currently', options = {}) ⇒ Object
1654 1655 1656 1657 1658 1659 1660 1661 1662 1663 1664 1665 1666 1667 1668 1669 1670 1671 1672 1673 1674 |
# File 'lib/doing/wwid.rb', line 1654 def archive(section = 'Currently', = {}) count = [:keep] || 0 destination = [:destination] || 'Archive' = [:tags] || [] bool = [:bool] || 'AND' section = choose_section if section.nil? || section =~ /choose/i archive_all = section =~ /^all$/i # && !(tags.nil? || tags.empty?) section = guess_section(section) unless archive_all add_section('Archive') if destination =~ /^archive$/i && !sections.include?('Archive') destination = guess_section(destination) if sections.include?(destination) && (sections.include?(section) || archive_all) do_archive(section, destination, { count: count, tags: , bool: bool, search: [:search], label: [:label] }) write(doing_file) else raise 'Either source or destination does not exist' end end |
#autotag(text) ⇒ Object
Does not repeat tags in a title, and only converts the first instance of an untagged keyword
1989 1990 1991 1992 1993 1994 1995 1996 1997 1998 1999 2000 2001 2002 2003 2004 2005 2006 2007 2008 2009 2010 2011 2012 2013 2014 2015 2016 2017 2018 2019 2020 2021 2022 2023 2024 2025 2026 2027 2028 2029 2030 2031 2032 2033 2034 2035 2036 2037 2038 2039 2040 2041 2042 2043 2044 2045 |
# File 'lib/doing/wwid.rb', line 1989 def autotag(text) return unless text return text unless @auto_tag = text.scan(/@\w+/) whitelisted = [] @config['autotag']['whitelist'].each do |tag| next if text =~ /@#{tag}\b/i text.sub!(/(?<!@)(#{tag.strip})\b/i) do |m| m.downcase! if tag =~ /[a-z]/ whitelisted.push("@#{m}") "@#{m}" end end = [] @config['autotag']['synonyms'].each do |tag, v| v.each do |word| next unless text =~ /\b#{word}\b/i .push(tag) unless .include?("@#{tag}") || whitelisted.include?("@#{tag}") end end if @config['autotag'].key? 'transform' @config['autotag']['transform'].each do |tag| next unless tag =~ /\S+:\S+/ rx, r = tag.split(/:/) r.gsub!(/\$/, '\\') rx.sub!(/^@/, '') regex = Regexp.new('@' + rx + '\b') matches = text.scan(regex) next unless matches matches.each do |m| new_tag = r if m.is_a?(Array) index = 1 m.each do |v| new_tag = new_tag.gsub('\\' + index.to_s, v) index += 1 end end .push(new_tag) end end end @results.push("Whitelisted tags: #{whitelisted.join(', ')}") if whitelisted.length > 0 if .length > 0 = .uniq.map { |t| '@' + t }.join(' ') @results.push("Synonym tags: #{}") text + ' ' + else text end end |
#choose_section ⇒ String
Returns The selected section name.
1229 1230 1231 1232 1233 1234 1235 1236 1237 1238 |
# File 'lib/doing/wwid.rb', line 1229 def choose_section sections.each_with_index do |section, i| puts format('% 3d: %s', i + 1, section) end print "#{colors['green']}> #{colors['default']}" num = STDIN.gets return false if num =~ /^[a-z ]*$/i sections[num.to_i - 1] end |
#choose_view ⇒ String
Returns The selected view name.
1254 1255 1256 1257 1258 1259 1260 1261 1262 1263 |
# File 'lib/doing/wwid.rb', line 1254 def choose_view views.each_with_index do |view, i| puts format('% 3d: %s', i + 1, view) end print '> ' num = STDIN.gets return false if num =~ /^[a-z ]*$/i views[num.to_i - 1] end |
#chronify(input) ⇒ DateTime
Returns result.
432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 449 450 451 |
# File 'lib/doing/wwid.rb', line 432 def chronify(input) now = Time.now raise "Invalid time expression #{input.inspect}" if input.to_s.strip == '' secs_ago = if input.match(/^(\d+)$/) # plain number, assume minutes Regexp.last_match(1).to_i * 60 elsif (m = input.match(/^(?:(?<day>\d+)d)?(?:(?<hour>\d+)h)?(?:(?<min>\d+)m)?$/i)) # day/hour/minute format e.g. 1d2h30m [[m['day'], 24 * 3600], [m['hour'], 3600], [m['min'], 60]].map { |qty, secs| qty ? (qty.to_i * secs) : 0 }.reduce(0, :+) end if secs_ago now - secs_ago else Chronic.parse(input, { context: :past, ambiguous_time_range: 8 }) end end |
#chronify_qty(qty) ⇒ Integer
Returns seconds.
462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 |
# File 'lib/doing/wwid.rb', line 462 def chronify_qty(qty) minutes = 0 case qty.strip when /^(\d+):(\d\d)$/ minutes += Regexp.last_match(1).to_i * 60 minutes += Regexp.last_match(2).to_i when /^(\d+(?:\.\d+)?)([hmd])?$/ amt = Regexp.last_match(1) type = Regexp.last_match(2).nil? ? 'm' : Regexp.last_match(2) minutes = case type.downcase when 'm' amt.to_i when 'h' (amt.to_f * 60).round when 'd' (amt.to_f * 60 * 24).round else minutes end end minutes * 60 end |
#colors ⇒ String
Returns ANSI escape sequence.
1754 1755 1756 1757 1758 1759 1760 1761 1762 1763 1764 1765 1766 1767 1768 1769 1770 1771 1772 1773 1774 1775 1776 1777 1778 1779 1780 1781 1782 1783 1784 1785 1786 1787 1788 1789 1790 1791 1792 1793 1794 1795 1796 |
# File 'lib/doing/wwid.rb', line 1754 def colors color = {} color['black'] = "\033[0;0;30m" color['red'] = "\033[0;0;31m" color['green'] = "\033[0;0;32m" color['yellow'] = "\033[0;0;33m" color['blue'] = "\033[0;0;34m" color['magenta'] = "\033[0;0;35m" color['cyan'] = "\033[0;0;36m" color['white'] = "\033[0;0;37m" color['bgblack'] = "\033[40m" color['bgred'] = "\033[41m" color['bggreen'] = "\033[42m" color['bgyellow'] = "\033[43m" color['bgblue'] = "\033[44m" color['bgmagenta'] = "\033[45m" color['bgcyan'] = "\033[46m" color['bgwhite'] = "\033[47m" color['boldblack'] = "\033[1;30m" color['boldred'] = "\033[1;31m" color['boldgreen'] = "\033[0;1;32m" color['boldyellow'] = "\033[0;1;33m" color['boldblue'] = "\033[0;1;34m" color['boldmagenta'] = "\033[0;1;35m" color['boldcyan'] = "\033[0;1;36m" color['boldwhite'] = "\033[0;1;37m" color['boldbgblack'] = "\033[1;40m" color['boldbgred'] = "\033[1;41m" color['boldbggreen'] = "\033[1;42m" color['boldbgyellow'] = "\033[1;43m" color['boldbgblue'] = "\033[1;44m" color['boldbgmagenta'] = "\033[1;45m" color['boldbgcyan'] = "\033[1;46m" color['boldbgwhite'] = "\033[1;47m" color['softpurple'] = "\033[0;35;40m" color['hotpants'] = "\033[7;34;40m" color['knightrider'] = "\033[7;30;40m" color['flamingo'] = "\033[7;31;47m" color['yeller'] = "\033[1;37;43m" color['whiteboard'] = "\033[1;30;47m" color['default'] = "\033[0;39m" color end |
#configure(opt = {}) ⇒ Object
170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 |
# File 'lib/doing/wwid.rb', line 170 def configure(opt = {}) @timers = {} opt[:ignore_local] ||= false @config_file ||= File.join(@user_home, @default_config_file) read_config({ ignore_local: opt[:ignore_local] }) @config = {} if @config.nil? @config['autotag'] ||= {} @config['autotag']['whitelist'] ||= [] @config['autotag']['synonyms'] ||= {} @config['doing_file'] ||= '~/what_was_i_doing.md' @config['current_section'] ||= 'Currently' @config['config_editor_app'] ||= nil @config['editor_app'] ||= nil @config['html_template'] ||= {} @config['html_template']['haml'] ||= nil @config['html_template']['css'] ||= nil @config['templates'] ||= {} @config['templates']['default'] ||= { 'date_format' => '%Y-%m-%d %H:%M', 'template' => '%date | %title%note', 'wrap_width' => 0 } @config['templates']['today'] ||= { 'date_format' => '%_I:%M%P', 'template' => '%date: %title %interval%note', 'wrap_width' => 0 } @config['templates']['last'] ||= { 'date_format' => '%-I:%M%P on %a', 'template' => '%title (at %date)%odnote', 'wrap_width' => 88 } @config['templates']['recent'] ||= { 'date_format' => '%_I:%M%P', 'template' => '%shortdate: %title (%section)', 'wrap_width' => 88 } @config['views'] ||= { 'done' => { 'date_format' => '%_I:%M%P', 'template' => '%date | %title%note', 'wrap_width' => 0, 'section' => 'All', 'count' => 0, 'order' => 'desc', 'tags' => 'done complete cancelled', 'tags_bool' => 'OR' }, 'color' => { 'date_format' => '%F %_I:%M%P', 'template' => '%boldblack%date %boldgreen| %boldwhite%title%default%note', 'wrap_width' => 0, 'section' => 'Currently', 'count' => 10, 'order' => 'asc' } } @config['marker_tag'] ||= 'flagged' @config['marker_color'] ||= 'red' @config['default_tags'] ||= [] @config['tag_sort'] ||= 'time' @current_section = config['current_section'] @default_template = config['templates']['default']['template'] @default_date_format = config['templates']['default']['date_format'] @config[:include_notes] ||= true # if ENV['DOING_DEBUG'].to_i == 3 # if @config['default_tags'].length > 0 # raise "DEFAULT CONFIG CHANGED" # end # end File.open(@config_file, 'w') { |yf| YAML.dump(@config, yf) } unless File.exist?(@config_file) @config = @local_config.deep_merge(@config) @current_section = @config['current_section'] @default_template = @config['templates']['default']['template'] @default_date_format = @config['templates']['default']['date_format'] end |
#create(filename = nil) ⇒ Object
340 341 342 343 344 345 346 347 |
# File 'lib/doing/wwid.rb', line 340 def create(filename = nil) filename = @doing_file if filename.nil? return if File.exist?(filename) && File.stat(filename).size.positive? File.open(filename, 'w+') do |f| f.puts "#{@current_section}:" end end |
#css_template ⇒ String
Returns CSS template.
333 334 335 |
# File 'lib/doing/wwid.rb', line 333 def css_template IO.read(File.join(File.dirname(__FILE__), '../templates/doing.css')) end |
#dedup(items, no_overlap = false) ⇒ Object
686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 |
# File 'lib/doing/wwid.rb', line 686 def dedup(items, no_overlap = false) combined = [] @content.each do |_k, v| combined += v['items'] end items.delete_if do |item| duped = false combined.each do |comp| duped = no_overlap ? overlapping_time?(item, comp) : same_time?(item, comp) break if duped end # warn "Skipping overlapping entry: #{item['title']}" if duped duped end end |
#do_archive(sect, destination, opt = {}) ⇒ Object
1683 1684 1685 1686 1687 1688 1689 1690 1691 1692 1693 1694 1695 1696 1697 1698 1699 1700 1701 1702 1703 1704 1705 1706 1707 1708 1709 1710 1711 1712 1713 1714 1715 1716 1717 1718 1719 1720 1721 1722 1723 1724 1725 1726 1727 1728 1729 1730 1731 1732 1733 1734 1735 1736 1737 1738 1739 1740 1741 1742 1743 1744 1745 1746 1747 |
# File 'lib/doing/wwid.rb', line 1683 def do_archive(sect, destination, opt = {}) count = opt[:count] || 0 = opt[:tags] || [] bool = opt[:bool] || 'AND' label = opt[:label] || true if sect =~ /^all$/i all_sections = sections.dup all_sections.delete(destination) else all_sections = [sect] end counter = 0 all_sections.each do |section| items = @content[section]['items'] moved_items = [] if !.empty? || opt[:search] items.delete_if do |item| if (!.empty? && item.(, bool) || (opt[:search] && item.matches_search?(opt[:search].to_s))) moved_items.push(item) counter += 1 true else false end end moved_items.each do |item| if label && section != 'Currently' item['title'] = item['title'].sub(/(?:@from\(.*?\))?(.*)$/, "\\1 @from(#{section})") end end @content[section]['items'] = items @content[destination]['items'].concat(moved_items) @results.push("Archived #{moved_items.length} items from #{section} to #{destination}") else count = items.length if count == 0 || items.length < count @content[section]['items'] = if count.zero? [] else items[0..count - 1] end items.map! do |item| if label && section != 'Currently' item['title'] = item['title'].sub(/(?:@from\(.*?\))?(.*)$/, "\\1 @from(#{section})") end item end if items.count > count @content[destination]['items'].concat(items[count..-1]) else @content[destination]['items'].concat(items) end @results.push("Archived #{items.length - count} items from #{section} to #{destination}") end end end |
#edit_last(section: 'All', options: {}) ⇒ Object
1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 1027 1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 1051 1052 1053 1054 1055 1056 1057 1058 1059 1060 1061 1062 1063 1064 1065 1066 1067 1068 1069 |
# File 'lib/doing/wwid.rb', line 1007 def edit_last(section: 'All', options: {}) section = guess_section(section) if section =~ /^all$/i items = [] @content.each do |_k, v| items.concat(v['items']) end # section = combined['items'].dup.sort_by { |item| item['date'] }.reverse[0]['section'] else items = @content[section]['items'] end items = items.sort_by { |item| item['date'] }.reverse idx = nil if [:tag] && ![:tag].empty? items.each_with_index do |item, i| if item.([:tag], [:tag_bool]) idx = i break end end elsif [:search] items.each_with_index do |item, i| if item.matches_search?([:search]) idx = i break end end else idx = 0 end if idx.nil? @results.push('No entries found') return end section = items[idx]['section'] section_items = @content[section]['items'] s_idx = section_items.index(items[idx]) current_item = section_items[s_idx]['title'] old_note = section_items[s_idx]['note'] ? section_items[s_idx]['note'].map(&:strip).join("\n") : nil current_item += "\n#{old_note}" unless old_note.nil? new_item = fork_editor(current_item) title, note = format_input(new_item) if title.nil? || title.empty? @results.push('No content provided') elsif title == section_items[s_idx]['title'] && note == old_note @results.push('No change in content') else section_items[s_idx]['title'] = title section_items[s_idx]['note'] = note @results.push("Entry edited: #{section_items[s_idx]['title']}") @content[section]['items'] = section_items write(@doing_file) end end |
#find_local_config ⇒ String
Returns A file path.
117 118 119 120 121 122 123 124 125 126 127 128 129 130 |
# File 'lib/doing/wwid.rb', line 117 def find_local_config config = {} dir = Dir.pwd local_config_files = [] while dir != '/' && (dir =~ %r{[A-Z]:/}).nil? local_config_files.push(File.join(dir, @default_config_file)) if File.exist? File.join(dir, @default_config_file) dir = File.dirname(dir) end local_config_files end |
#fork_editor(input = '') ⇒ Object
354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 |
# File 'lib/doing/wwid.rb', line 354 def fork_editor(input = '') tmpfile = Tempfile.new(['doing', '.md']) File.open(tmpfile.path, 'w+') do |f| f.puts input f.puts "\n# The first line is the entry title, any lines after that are added as a note" end pid = Process.fork { system("$EDITOR #{tmpfile.path}") } trap('INT') do begin Process.kill(9, pid) rescue StandardError Errno::ESRCH end tmpfile.unlink tmpfile.close! exit 0 end Process.wait(pid) begin if $?.exitstatus == 0 input = IO.read(tmpfile.path) else raise 'Cancelled' end ensure tmpfile.close tmpfile.unlink end input.split(/\n/).delete_if {|line| line =~ /^#/ }.join("\n").strip end |
#format_input(input) ⇒ Array
Returns [(String)title, (Array)note].
398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 |
# File 'lib/doing/wwid.rb', line 398 def format_input(input) raise 'No content in entry' if input.nil? || input.strip.empty? input_lines = input.split(/[\n\r]+/).delete_if {|line| line =~ /^#/ || line =~ /^\s*$/ } title = input_lines[0]&.strip raise 'No content in first line' if title.nil? || title.strip.empty? note = input_lines.length > 1 ? input_lines[1..-1] : [] # If title line ends in a parenthetical, use that as the note if note.empty? && title =~ /\(.*?\)$/ title.sub!(/\((.*?)\)$/) do m = Regexp.last_match note.push(m[1]) '' end end note.map!(&:strip) note.delete_if { |line| line =~ /^\s*$/ || line =~ /^#/ } [title, note] end |
#get_view(title) ⇒ Object
1270 1271 1272 1273 1274 |
# File 'lib/doing/wwid.rb', line 1270 def get_view(title) return @config['views'][title] if @config['views'].has_key?(title) false end |
#guess_section(frag, guessed: false) ⇒ Object
511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 |
# File 'lib/doing/wwid.rb', line 511 def guess_section(frag, guessed: false) return 'All' if frag =~ /all/i sections.each { |section| return section.cap_first if frag.downcase == section.downcase } section = false re = frag.split('').join('.*?') sections.each do |sect| next unless sect =~ /#{re}/i warn "Assuming you meant #{sect}" section = sect break end unless section || guessed alt = guess_view(frag, true) raise "Did you mean `doing view #{alt}`?" if alt res = yn("Section #{frag} not found, create it", default_response: false) if res add_section(frag.cap_first) write(@doing_file) return frag.cap_first end raise "Unknown section: #{frag}" end section ? section.cap_first : guessed end |
#guess_view(frag, guessed = false) ⇒ Object
592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 |
# File 'lib/doing/wwid.rb', line 592 def guess_view(frag, guessed = false) views.each { |view| return view if frag.downcase == view.downcase } view = false re = frag.split('').join('.*?') views.each do |v| next unless v =~ /#{re}/i warn "Assuming you meant #{v}" view = v break end unless view || guessed alt = guess_section(frag, guessed: true) if alt raise "Did you mean `doing show #{alt}`?" else raise "Unknown view: #{frag}" end end view end |
#haml_template ⇒ String
Returns HAML template.
324 325 326 |
# File 'lib/doing/wwid.rb', line 324 def haml_template IO.read(File.join(File.dirname(__FILE__), '../templates/doing.haml')) end |
#import_timing(path, opt = {}) ⇒ Object
711 712 713 714 715 716 717 718 719 720 721 722 723 724 725 726 727 728 729 730 731 732 733 734 735 736 737 738 739 740 741 742 743 744 745 746 747 748 749 750 751 752 753 754 755 756 757 758 |
# File 'lib/doing/wwid.rb', line 711 def import_timing(path, opt = {}) section = opt[:section] || @current_section opt[:no_overlap] ||= false add_section(section) unless @content.has_key?(section) = opt[:tag] ? opt[:tag].split(/[ ,]+/).map { |t| t.sub(/^@?/, '@') }.join(' ') : '' prefix = opt[:prefix] ? opt[:prefix] : '[Timing.app]' raise "File not found" unless File.exist?(File.(path)) data = JSON.parse(IO.read(File.(path))) new_items = [] data.each do |entry| # Only process task entries next if entry.key?('activityType') && entry['activityType'] != 'Task' # Only process entries with a start and end date next unless entry.key?('startDate') && entry.key?('endDate') # Round down seconds and convert UTC to local time start_time = Time.parse(entry['startDate'].sub(/:\d\dZ$/, ':00Z')).getlocal end_time = Time.parse(entry['endDate'].sub(/:\d\dZ$/, ':00Z')).getlocal next unless start_time && end_time = entry['project'].split(/ ▸ /).map {|proj| proj.gsub(/[^a-z0-9]+/i, '').downcase } title = "#{prefix} " title += entry.key?('activityTitle') && entry['activityTitle'] != '(Untitled Task)' ? entry['activityTitle'] : 'Working on' .each do |tag| if title =~ /\b#{tag}\b/i title.sub!(/\b#{tag}\b/i, "@#{tag}") else title += " @#{tag}" end end title = autotag(title) if @auto_tag title += " @done(#{end_time.strftime('%Y-%m-%d %H:%M')})" title.gsub!(/ +/, ' ') title.strip! new_entry = { 'title' => title, 'date' => start_time, 'section' => section } new_entry['note'] = entry['notes'].split(/\n/).map(&:chomp) if entry.key?('notes') new_items.push(new_entry) end total = new_items.count new_items = dedup(new_items, opt[:no_overlap]) dups = total - new_items.count @results.push(%(Skipped #{dups} items with overlapping times)) if dups > 0 @content[section]['items'].concat(new_items) @results.push(%(Imported #{new_items.count} items to #{section})) end |
#init_doing_file(path = nil) ⇒ Object
264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 |
# File 'lib/doing/wwid.rb', line 264 def init_doing_file(path = nil) @doing_file = File.(@config['doing_file']) input = path if input.nil? create(@doing_file) unless File.exist?(@doing_file) input = IO.read(@doing_file) input = input.force_encoding('utf-8') if input.respond_to? :force_encoding elsif File.exist?(File.(input)) && File.file?(File.(input)) && File.stat(File.(input)).size.positive? @doing_file = File.(input) input = IO.read(File.(input)) input = input.force_encoding('utf-8') if input.respond_to? :force_encoding elsif input.length < 256 @doing_file = File.(input) create(input) input = IO.read(File.(input)) input = input.force_encoding('utf-8') if input.respond_to? :force_encoding end @other_content_top = [] @other_content_bottom = [] section = 'Uncategorized' lines = input.split(/[\n\r]/) current = 0 lines.each do |line| next if line =~ /^\s*$/ if line =~ /^(\S[\S ]+):\s*(@\S+\s*)*$/ section = Regexp.last_match(1) @content[section] = {} @content[section]['original'] = line @content[section]['items'] = [] current = 0 elsif line =~ /^\s*- (\d{4}-\d\d-\d\d \d\d:\d\d) \| (.*)/ date = Time.parse(Regexp.last_match(1)) title = Regexp.last_match(2) @content[section]['items'].push({ 'title' => title, 'date' => date, 'section' => section }) current += 1 elsif current.zero? # if content[section]['items'].length - 1 == current @other_content_top.push(line) elsif line =~ /^\S/ @other_content_bottom.push(line) else @content[section]['items'][current - 1]['note'] = [] unless @content[section]['items'][current - 1].key? 'note' @content[section]['items'][current - 1]['note'].push(line.chomp) # end end end end |
#last(times: true, section: nil, options: {}) ⇒ Object
1878 1879 1880 1881 1882 1883 1884 1885 1886 1887 1888 1889 1890 1891 1892 1893 1894 1895 1896 1897 1898 1899 1900 1901 |
# File 'lib/doing/wwid.rb', line 1878 def last(times: true, section: nil, options: {}) section = section.nil? || section =~ /all/i ? 'All' : guess_section(section) cfg = @config['templates']['last'] opts = { section: section, wrap_width: cfg['wrap_width'], count: 1, format: cfg['date_format'], template: cfg['template'], times: times } if [:tag] opts[:tag_filter] = { 'tags' => [:tag], 'bool' => [:tag_bool] } end opts[:search] = [:search] if [:search] list_section(opts) end |
#last_entry(opt = {}) ⇒ Object
819 820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 |
# File 'lib/doing/wwid.rb', line 819 def last_entry(opt = {}) opt[:tag_bool] ||= 'AND' opt[:section] ||= @current_section sec_arr = [] if opt[:section].nil? sec_arr = [@current_section] elsif opt[:section].instance_of?(String) if opt[:section] =~ /^all$/i combined = { 'items' => [] } @content.each do |_k, v| combined['items'] += v['items'] end items = combined['items'].dup.sort_by { |item| item['date'] }.reverse sec_arr.push(items[0]['section']) else sec_arr = [guess_section(opt[:section])] end end all_items = [] sec_arr.each do |section| all_items.concat(@content[section]['items'].dup) if @content.key?(section) end if opt[:tag] && opt[:tag].length.positive? all_items.select! { |item| item.(opt[:tag], opt[:tag_bool]) } elsif opt[:search]&.length all_items.select! { |item| item.matches_search?(opt[:search]) } end all_items.max_by { |item| item['date'] } end |
#last_note(section = 'All') ⇒ Object
766 767 768 769 770 771 772 773 774 775 776 777 778 779 780 781 782 783 |
# File 'lib/doing/wwid.rb', line 766 def last_note(section = 'All') section = guess_section(section) if section =~ /^all$/i combined = { 'items' => [] } @content.each do |_k, v| combined['items'] += v['items'] end section = combined['items'].dup.sort_by { |item| item['date'] }.reverse[0]['section'] end raise "Section #{section} not found" unless @content.key?(section) last_item = @content[section]['items'].dup.sort_by { |item| item['date'] }.reverse[0] warn "Editing note for #{last_item['title']}" note = '' note = last_item['note'].map(&:strip).join("\n") unless last_item['note'].nil? "#{last_item['title']}\n# EDIT BELOW THIS LINE ------------\n#{note}" end |
#list_date(dates, section, times = nil, output = nil, opt = {}) ⇒ Object
1823 1824 1825 1826 1827 1828 1829 1830 1831 1832 |
# File 'lib/doing/wwid.rb', line 1823 def list_date(dates, section, times = nil, output = nil, opt = {}) opt[:totals] ||= false opt[:sort_tags] ||= false section = guess_section(section) # :date_filter expects an array with start and end date dates = [dates, dates] if dates.instance_of?(String) list_section({ section: section, count: 0, order: 'asc', date_filter: dates, times: times, output: output, totals: opt[:totals], sort_tags: opt[:sort_tags] }) end |
#list_section(opt = {}) ⇒ Object
1282 1283 1284 1285 1286 1287 1288 1289 1290 1291 1292 1293 1294 1295 1296 1297 1298 1299 1300 1301 1302 1303 1304 1305 1306 1307 1308 1309 1310 1311 1312 1313 1314 1315 1316 1317 1318 1319 1320 1321 1322 1323 1324 1325 1326 1327 1328 1329 1330 1331 1332 1333 1334 1335 1336 1337 1338 1339 1340 1341 1342 1343 1344 1345 1346 1347 1348 1349 1350 1351 1352 1353 1354 1355 1356 1357 1358 1359 1360 1361 1362 1363 1364 1365 1366 1367 1368 1369 1370 1371 1372 1373 1374 1375 1376 1377 1378 1379 1380 1381 1382 1383 1384 1385 1386 1387 1388 1389 1390 1391 1392 1393 1394 1395 1396 1397 1398 1399 1400 1401 1402 1403 1404 1405 1406 1407 1408 1409 1410 1411 1412 1413 1414 1415 1416 1417 1418 1419 1420 1421 1422 1423 1424 1425 1426 1427 1428 1429 1430 1431 1432 1433 1434 1435 1436 1437 1438 1439 1440 1441 1442 1443 1444 1445 1446 1447 1448 1449 1450 1451 1452 1453 1454 1455 1456 1457 1458 1459 1460 1461 1462 1463 1464 1465 1466 1467 1468 1469 1470 1471 1472 1473 1474 1475 1476 1477 1478 1479 1480 1481 1482 1483 1484 1485 1486 1487 1488 1489 1490 1491 1492 1493 1494 1495 1496 1497 1498 1499 1500 1501 1502 1503 1504 1505 1506 1507 1508 1509 1510 1511 1512 1513 1514 1515 1516 1517 1518 1519 1520 1521 1522 1523 1524 1525 1526 1527 1528 1529 1530 1531 1532 1533 1534 1535 1536 1537 1538 1539 1540 1541 1542 1543 1544 1545 1546 1547 1548 1549 1550 1551 1552 1553 1554 1555 1556 1557 1558 1559 1560 1561 1562 1563 1564 1565 1566 1567 1568 1569 1570 1571 1572 1573 1574 1575 1576 1577 1578 1579 1580 1581 1582 1583 1584 1585 1586 1587 1588 1589 1590 1591 1592 1593 1594 1595 1596 1597 1598 1599 1600 1601 1602 1603 1604 1605 1606 1607 1608 1609 1610 1611 1612 1613 1614 1615 1616 1617 1618 1619 1620 1621 1622 1623 1624 1625 1626 1627 1628 1629 1630 1631 1632 1633 1634 1635 1636 1637 1638 1639 1640 1641 1642 1643 1644 1645 |
# File 'lib/doing/wwid.rb', line 1282 def list_section(opt = {}) opt[:count] ||= 0 count = opt[:count] - 1 opt[:section] ||= nil opt[:format] ||= @default_date_format opt[:template] ||= @default_template opt[:age] ||= 'newest' opt[:order] ||= 'desc' opt[:today] ||= false opt[:tag_filter] ||= false opt[:tags_color] ||= false opt[:times] ||= false opt[:totals] ||= false opt[:sort_tags] ||= false opt[:search] ||= false opt[:only_timed] ||= false opt[:date_filter] ||= [] # opt[:highlight] ||= true section = '' if opt[:section].nil? section = choose_section opt[:section] = @content[section] elsif opt[:section].instance_of?(String) if opt[:section] =~ /^all$/i combined = { 'items' => [] } @content.each do |_k, v| combined['items'] += v['items'] end section = if opt[:tag_filter] && opt[:tag_filter]['bool'] != 'NONE' opt[:tag_filter]['tags'].map do |tag| "@#{tag}" end.join(' + ') else 'doing' end opt[:section] = combined else section = guess_section(opt[:section]) opt[:section] = @content[section] end end raise 'Invalid section object' unless opt[:section].instance_of? Hash items = opt[:section]['items'].sort_by { |item| item['date'] } if opt[:date_filter].length == 2 start_date = opt[:date_filter][0] end_date = opt[:date_filter][1] items.keep_if do |item| if end_date item['date'] >= start_date && item['date'] <= end_date else item['date'].strftime('%F') == start_date.strftime('%F') end end end if opt[:tag_filter] && !opt[:tag_filter]['tags'].empty? items.delete_if do |item| case opt[:tag_filter]['bool'] when /(AND|ALL)/ del = false opt[:tag_filter]['tags'].each do |tag| unless item['title'] =~ /@#{tag}/ del = true break end end del when /NONE/ del = false opt[:tag_filter]['tags'].each do |tag| del = true if item['title'] =~ /@#{tag}/ end del when /(OR|ANY)/ del = true opt[:tag_filter]['tags'].each do |tag| del = false if item['title'] =~ /@#{tag}/ end del end end end if opt[:search] items.keep_if {|item| item.matches_search?(opt[:search]) } end if opt[:only_timed] items.delete_if do |item| get_interval(item) == false end end if opt[:today] items.delete_if do |item| item['date'] < Date.today.to_time end.reverse! section = Time.now.strftime('%A, %B %d') elsif opt[:yesterday] items.delete_if do |item| item['date'] <= Date.today.prev_day.to_time or item['date'] >= Date.today.to_time end.reverse! elsif opt[:age] =~ /oldest/i items = items[0..count] else items = items.reverse[0..count] end items.reverse! if opt[:order] =~ /^a/i out = '' raise 'Unknown output format' if opt[:output] && (opt[:output] !~ /^(template|html|csv|json|timeline)$/i) case opt[:output] when /^csv$/i output = [CSV.generate_line(%w[date title note timer section])] items.each do |i| note = '' if i['note'] arr = i['note'].map { |line| line.strip }.delete_if { |e| e =~ /^\s*$/ } note = arr.join("\n") unless arr.nil? end interval = get_interval(i, false) if i['title'] =~ /@done\((\d{4}-\d\d-\d\d \d\d:\d\d.*?)\)/ && opt[:times] interval ||= 0 output.push(CSV.generate_line([i['date'], i['title'], note, interval, i['section']])) end out = output.join('') when /^(json|timeline)/i items_out = [] max = items[-1]['date'].strftime('%F') min = items[0]['date'].strftime('%F') items.each_with_index do |i, index| if String.method_defined? :force_encoding title = i['title'].force_encoding('utf-8') note = i['note'].map { |line| line.force_encoding('utf-8').strip } if i['note'] else title = i['title'] note = i['note'].map { |line| line.strip } if i['note'] end if i['title'] =~ /@done\((\d{4}-\d\d-\d\d \d\d:\d\d.*?)\)/ && opt[:times] end_date = Time.parse(Regexp.last_match(1)) interval = get_interval(i, false) end end_date ||= '' interval ||= 0 note ||= '' = [] = %w[meanwhile done cancelled flagged] i['title'].scan(/@([^(\s]+)(?:\((.*?)\))?/).each do |tag| .push(tag[0]) unless .include?(tag[0]) end if opt[:output] == 'json' items_out << { date: i['date'], end_date: end_date, title: title.strip, #+ " #{note}" note: note.instance_of?(Array) ? note.map(&:strip).join("\n") : note, time: '%02d:%02d:%02d' % fmt_time(interval), tags: } elsif opt[:output] == 'timeline' new_item = { 'id' => index + 1, 'content' => title.strip, #+ " #{note}" 'title' => title.strip + " (#{'%02d:%02d:%02d' % fmt_time(interval)})", 'start' => i['date'].strftime('%F'), 'type' => 'point' } if interval && interval > 0 new_item['end'] = end_date.strftime('%F') new_item['type'] = 'range' if interval > 3600 * 3 end items_out.push(new_item) end end if opt[:output] == 'json' out = { 'section' => section, 'items' => items_out, 'timers' => tag_times('json', opt[:sort_tags]) }.to_json elsif opt[:output] == 'timeline' template = <<~EOTEMPLATE <!doctype html> <html> <head> <link href="http://visjs.org/dist/vis.css" rel="stylesheet" type="text/css" /> <script src="http://visjs.org/dist/vis.js"></script> </head> <body> <div id="mytimeline"></div> #{' '} <script type="text/javascript"> // DOM element where the Timeline will be attached var container = document.getElementById('mytimeline'); #{' '} // Create a DataSet with data (enables two way data binding) var data = new vis.DataSet(#{items_out.to_json}); #{' '} // Configuration for the Timeline var options = { width: '100%', height: '800px', margin: { item: 20 }, stack: true, min: '#{min}', max: '#{max}' }; #{' '} // Create a Timeline var timeline = new vis.Timeline(container, data, options); </script> </body> </html> EOTEMPLATE return template end when /^html$/i page_title = section items_out = [] items.each do |i| # if i.has_key?('note') # note = '<span class="note">' + i['note'].map{|n| n.strip }.join('<br>') + '</span>' # else # note = '' # end if String.method_defined? :force_encoding title = i['title'].force_encoding('utf-8').link_urls note = i['note'].map { |line| line.force_encoding('utf-8').strip.link_urls } if i['note'] else title = i['title'].link_urls note = i['note'].map { |line| line.strip.link_urls } if i['note'] end interval = get_interval(i) if i['title'] =~ /@done\((\d{4}-\d\d-\d\d \d\d:\d\d.*?)\)/ && opt[:times] interval ||= false items_out << { date: i['date'].strftime('%a %-I:%M%p'), title: title.gsub(/(@[^ (]+(\(.*?\))?)/im, '<span class="tag">\1</span>').strip, #+ " #{note}" note: note, time: interval, section: i['section'] } end template = if @config['html_template']['haml'] && File.exist?(File.(@config['html_template']['haml'])) IO.read(File.(@config['html_template']['haml'])) else haml_template end style = if @config['html_template']['css'] && File.exist?(File.(@config['html_template']['css'])) IO.read(File.(@config['html_template']['css'])) else css_template end totals = opt[:totals] ? tag_times('html', opt[:sort_tags]) : '' engine = Haml::Engine.new(template) puts engine.render(Object.new, { :@items => items_out, :@page_title => page_title, :@style => style, :@totals => totals }) else items.each do |item| if opt[:highlight] && item['title'] =~ /@#{@config['marker_tag']}\b/i flag = colors[@config['marker_color']] reset = colors['default'] else flag = '' reset = '' end if (item.has_key?('note') && !item['note'].empty?) && @config[:include_notes] note_lines = item['note'].delete_if do |line| line =~ /^\s*$/ end.map { |line| "\t\t" + line.sub(/^\t*/, '').sub(/^-/, '—') + ' ' } if opt[:wrap_width] && opt[:wrap_width] > 0 width = opt[:wrap_width] note_lines.map! do |line| line.strip.gsub(/(.{1,#{width}})(\s+|\Z)/, "\t\\1\n") end end note = "\n#{note_lines.join("\n").chomp}" else note = '' end output = opt[:template].dup output.gsub!(/%[a-z]+/) do |m| if colors.has_key?(m.sub(/^%/, '')) colors[m.sub(/^%/, '')] else m end end output.sub!(/%date/, item['date'].strftime(opt[:format])) interval = get_interval(item) if item['title'] =~ /@done\((\d{4}-\d\d-\d\d \d\d:\d\d.*?)\)/ && opt[:times] interval ||= '' output.sub!(/%interval/, interval) output.sub!(/%shortdate/) do if item['date'] > Date.today.to_time item['date'].strftime('%_I:%M%P') elsif item['date'] > (Date.today - 7).to_time item['date'].strftime('%a %-I:%M%P') elsif item['date'].year == Date.today.year item['date'].strftime('%b %d, %-I:%M%P') else item['date'].strftime('%b %d %Y, %-I:%M%P') end end output.sub!(/%title/) do |_m| if opt[:wrap_width] && opt[:wrap_width] > 0 flag + item['title'].gsub(/(.{1,#{opt[:wrap_width]}})(\s+|\Z)/, "\\1\n\t ").chomp + reset else flag + item['title'].chomp + reset end end output.sub!(/%section/, item['section']) if item['section'] if opt[:tags_color] escapes = output.scan(/(\e\[[\d;]+m)[^\e]+@/) last_color = if escapes.length > 0 escapes[-1][0] else colors['default'] end output.gsub!(/(\s|m)(@[^ (]+)/, "\\1#{colors[opt[:tags_color]]}\\2#{last_color}") end output.sub!(/%note/, note) output.sub!(/%odnote/, note.gsub(/^\t*/, '')) output.sub!(/%chompnote/, note.gsub(/\n+/, ' ').gsub(/(^\s*|\s*$)/, '').gsub(/\s+/, ' ')) output.gsub!(/%hr(_under)?/) do |_m| o = '' `tput cols`.to_i.times do o += Regexp.last_match(1).nil? ? '-' : '_' end o end output.gsub!(/%n/, "\n") output.gsub!(/%t/, "\t") out += "#{output}\n" end out += tag_times('text', opt[:sort_tags]) if opt[:totals] end out end |
#note_last(section, note, replace: false) ⇒ Object
1078 1079 1080 1081 1082 1083 1084 1085 1086 1087 1088 1089 1090 1091 1092 1093 1094 1095 1096 1097 1098 1099 1100 1101 1102 1103 1104 1105 1106 1107 1108 1109 1110 1111 1112 1113 1114 1115 1116 1117 1118 |
# File 'lib/doing/wwid.rb', line 1078 def note_last(section, note, replace: false) section = guess_section(section) if section =~ /^all$/i combined = { 'items' => [] } @content.each do |_k, v| combined['items'] += v['items'] end section = combined['items'].dup.sort_by { |item| item['date'] }.reverse[0]['section'] end raise "Section #{section} not found" unless @content.key?(section) # sort_section(opt[:section]) items = @content[section]['items'].dup.sort_by { |item| item['date'] }.reverse current_note = items[0]['note'] current_note = [] if current_note.nil? title = items[0]['title'] if replace items[0]['note'] = note if note.empty? && !current_note.empty? @results.push(%(Removed note from "#{title}")) elsif !current_note.empty? && !note.empty? @results.push(%(Replaced note from "#{title}")) elsif !note.empty? @results.push(%(Added note to "#{title}")) else @results.push(%(Entry "#{title}" has no note)) end elsif current_note.instance_of?(Array) items[0]['note'] = current_note.concat(note) @results.push(%(Added note to "#{title}")) unless note.empty? else items[0]['note'] = note @results.push(%(Added note to "#{title}")) unless note.empty? end @content[section]['items'] = items end |
#overlapping_time?(item_a, item_b) ⇒ Boolean
674 675 676 677 678 679 680 681 682 683 684 |
# File 'lib/doing/wwid.rb', line 674 def overlapping_time?(item_a, item_b) return true if same_time?(item_a, item_b) start_a = item_a['date'] interval = get_interval(item_a, false) end_a = interval ? start_a + interval.to_i : start_a start_b = item_b['date'] interval = get_interval(item_b, false) end_b = interval ? start_b + interval.to_i : start_b (start_a >= start_b && start_a <= end_b) || (end_a >= start_b && end_a <= end_b) || (start_a < start_b && end_a > end_b) end |
#read_config(opt = {}) ⇒ Object
135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 |
# File 'lib/doing/wwid.rb', line 135 def read_config(opt = {}) @config_file ||= if Dir.respond_to?('home') File.join(Dir.home, @default_config_file) else File.join(File.('~'), @default_config_file) end additional_configs = if opt[:ignore_local] [] else find_local_config end begin @local_config = {} @config = YAML.load_file(@config_file) || {} if File.exist?(@config_file) additional_configs.each do |cfg| new_config = YAML.load_file(cfg) || {} if cfg @local_config = @local_config.deep_merge(new_config) end # @config.deep_merge(@local_config) rescue StandardError @config = {} @local_config = {} # raise "error reading config" end end |
#recent(count = 10, section = nil, opt = {}) ⇒ Object
1857 1858 1859 1860 1861 1862 1863 1864 1865 1866 1867 1868 1869 1870 |
# File 'lib/doing/wwid.rb', line 1857 def recent(count = 10, section = nil, opt = {}) times = opt[:t] || true opt[:totals] ||= false opt[:sort_tags] ||= false cfg = @config['templates']['recent'] section ||= @current_section section = guess_section(section) list_section({ section: section, wrap_width: cfg['wrap_width'], count: count, format: cfg['date_format'], template: cfg['template'], order: 'asc', times: times, totals: opt[:totals], sort_tags: opt[:sort_tags], tags_color: opt[:tags_color] }) end |
#restart_last(opt = {}) ⇒ Object
790 791 792 793 794 795 796 797 798 799 800 801 802 803 804 805 806 807 808 809 810 811 812 |
# File 'lib/doing/wwid.rb', line 790 def restart_last(opt = {}) opt[:section] ||= 'all' opt[:note] ||= [] opt[:tag] ||= [] opt[:tag_bool] ||= 'AND' last = last_entry(opt) if last.nil? @results.push(%(No previous entry found)) return end unless last.(['done'], 'ALL') new_item = last.dup new_item['title'] += " @done(#{Time.now.strftime('%F %R')})" update_item(last, new_item) end # Remove @done tag title = last['title'].sub(/\s*@done(\(.*?\))?/, '').chomp section = opt[:in].nil? ? last['section'] : guess_section(opt[:in]) @auto_tag = false add_item(title, section, { note: opt[:note], back: opt[:date], timed: true }) write(@doing_file) end |
#restore_backup(file) ⇒ Object
1216 1217 1218 1219 1220 1221 1222 |
# File 'lib/doing/wwid.rb', line 1216 def restore_backup(file) if File.exist?(file + '~') puts file + '~' FileUtils.cp(file + '~', file) @results.push("Restored #{file}") end end |
#same_time?(item_a, item_b) ⇒ Boolean
670 671 672 |
# File 'lib/doing/wwid.rb', line 670 def same_time?(item_a, item_b) item_a['date'] == item_b['date'] ? get_interval(item_a, false) == get_interval(item_b, false) : false end |
#stop_start(target_tag, opt = {}) ⇒ Object
1130 1131 1132 1133 1134 1135 1136 1137 1138 1139 1140 1141 1142 1143 1144 1145 1146 1147 1148 1149 1150 1151 1152 1153 1154 1155 1156 1157 1158 1159 1160 1161 1162 1163 1164 1165 1166 1167 1168 1169 1170 1171 1172 1173 |
# File 'lib/doing/wwid.rb', line 1130 def stop_start(target_tag, opt = {}) tag = target_tag.dup opt[:section] ||= @current_section opt[:archive] ||= false opt[:back] ||= Time.now opt[:new_item] ||= false opt[:note] ||= false opt[:section] = guess_section(opt[:section]) tag.sub!(/^@/, '') found_items = 0 @content[opt[:section]]['items'].each_with_index do |item, i| next unless item['title'] =~ /@#{tag}/ title = item['title'].gsub(/(^| )@(#{tag}|done)(\([^)]*\))?/, '') title += " @done(#{opt[:back].strftime('%F %R')})" @content[opt[:section]]['items'][i]['title'] = title found_items += 1 if opt[:archive] && opt[:section] != 'Archive' @results.push(%(Completed and archived "#{@content[opt[:section]]['items'][i]['title']}")) archive_item = @content[opt[:section]]['items'][i] archive_item['title'] = i['title'].sub(/(?:@from\(.*?\))?(.*)$/, "\\1 @from(#{i['section']})") @content['Archive']['items'].push(archive_item) @content[opt[:section]]['items'].delete_at(i) else @results.push(%(Completed "#{@content[opt[:section]]['items'][i]['title']}")) end end @results.push("No active @#{tag} tasks found.") if found_items == 0 if opt[:new_item] title, note = format_input(opt[:new_item]) note.push(opt[:note].map(&:chomp)) if opt[:note] title += " @#{tag}" add_item(title.cap_first, opt[:section], { note: note.join(' ').rstrip, back: opt[:back] }) end write(@doing_file) end |
#tag_last(opt = {}) ⇒ Object
859 860 861 862 863 864 865 866 867 868 869 870 871 872 873 874 875 876 877 878 879 880 881 882 883 884 885 886 887 888 889 890 891 892 893 894 895 896 897 898 899 900 901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 953 954 955 956 957 958 959 960 961 962 963 964 965 966 967 968 969 970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 |
# File 'lib/doing/wwid.rb', line 859 def tag_last(opt = {}) opt[:section] ||= @current_section opt[:count] ||= 1 opt[:archive] ||= false opt[:tags] ||= ['done'] opt[:sequential] ||= false opt[:date] ||= false opt[:remove] ||= false opt[:autotag] ||= false opt[:back] ||= false opt[:took] ||= nil sec_arr = [] if opt[:section].nil? sec_arr = [@current_section] elsif opt[:section].instance_of?(String) if opt[:section] =~ /^all$/i if opt[:count] == 1 combined = { 'items' => [] } @content.each do |_k, v| combined['items'] += v['items'] end items = combined['items'].dup.sort_by { |item| item['date'] }.reverse sec_arr.push(items[0]['section']) elsif opt[:count] > 1 raise 'A count greater than one requires a section to be specified' else sec_arr = sections end else sec_arr = [guess_section(opt[:section])] end end sec_arr.each do |section| if @content.key?(section) items = @content[section]['items'].dup.sort_by { |item| item['date'] }.reverse idx = 0 done_date = Time.now next_start = Time.now count = (opt[:count]).zero? ? items.length : opt[:count] items.map! do |item| break if idx == count tag_match = opt[:tag].nil? || opt[:tag].empty? ? true : item.(opt[:tag], opt[:tag_bool]) search_match = opt[:search].nil? || opt[:search].empty? ? true : item.matches_search?(opt[:search]) if tag_match && search_match if opt[:autotag] new_title = autotag(item['title']) if @auto_tag if new_title == item['title'] @results.push(%(Autotag: No changes)) else @results.push("Tags updated: #{new_title}") item['title'] = new_title end else if opt[:sequential] done_date = next_start - 1 next_start = item['date'] elsif opt[:back] if opt[:back].is_a? Integer done_date = item['date'] + opt[:back] else done_date = item['date'] + (opt[:back] - item['date']) end elsif opt[:took] if item['date'] + opt[:took] > Time.now item['date'] = Time.now - opt[:took] done_date = Time.now else done_date = item['date'] + opt[:took] end else done_date = Time.now end title = item['title'] opt[:tags].each do |tag| tag = tag.strip if opt[:remove] if title =~ /@#{tag}\b/ title.gsub!(/(^| )@#{tag}(\([^)]*\))?/, '') @results.push(%(Removed @#{tag}: "#{title}" in #{section})) end elsif title !~ /@#{tag}/ title.chomp! title += if opt[:date] " @#{tag}(#{done_date.strftime('%F %R')})" else " @#{tag}" end @results.push(%(Added @#{tag}: "#{title}" in #{section})) end end item['title'] = title end idx += 1 end item end @content[section]['items'] = items if opt[:archive] && section != 'Archive' && (opt[:count]).positive? # concat [count] items from [section] and archive section archived = @content[section]['items'][0..opt[:count] - 1].map do |i| i['title'].sub!(/(?:@from\(.*?\))?(.*)$/, "\\1 @from(#{i['section']})") i end.concat(@content['Archive']['items']) # slice [count] items off of [section] items @content[opt[:section]]['items'] = @content[opt[:section]]['items'][opt[:count]..-1] # overwrite archive section with concatenated array @content['Archive']['items'] = archived # log it result = opt[:count] == 1 ? '1 entry' : "#{opt[:count]} entries" @results.push("Archived #{result} from #{section}") elsif opt[:archive] && (opt[:count]).zero? @results.push('Archiving is skipped when operating on all entries') if (opt[:count]).zero? end else raise "Section not found: #{section}" end end write(@doing_file) end |
#tag_times(format = 'text', sort_by_name = false) ⇒ Object
1908 1909 1910 1911 1912 1913 1914 1915 1916 1917 1918 1919 1920 1921 1922 1923 1924 1925 1926 1927 1928 1929 1930 1931 1932 1933 1934 1935 1936 1937 1938 1939 1940 1941 1942 1943 1944 1945 1946 1947 1948 1949 1950 1951 1952 1953 1954 1955 1956 1957 1958 1959 1960 1961 1962 1963 1964 1965 1966 1967 1968 1969 1970 1971 1972 1973 1974 1975 1976 1977 1978 1979 1980 1981 |
# File 'lib/doing/wwid.rb', line 1908 def tag_times(format = 'text', sort_by_name = false) return '' if @timers.empty? max = @timers.keys.sort_by { |k| k.length }.reverse[0].length + 1 total = @timers.delete('All') = @timers.delete_if { |_k, v| v == 0 } = if sort_by_name .sort_by { |k, _v| k }.reverse else .sort_by { |_k, v| v } end if format == 'html' output = <<EOS <table> <caption id="tagtotals">Tag Totals</caption> <colgroup> <col style="text-align:left;"/> <col style="text-align:left;"/> </colgroup> <thead> <tr> <th style="text-align:left;">project</th> <th style="text-align:left;">time</th> </tr> </thead> <tbody> EOS .reverse.each do |k, v| if v > 0 output += "<tr><td style='text-align:left;'>#{k}</td><td style='text-align:left;'>#{'%02d:%02d:%02d' % fmt_time(v)}</td></tr>\n" end end tail = <<EOS <tr> <td style="text-align:left;" colspan="2"></td> </tr> </tbody> <tfoot> <tr> <td style="text-align:left;"><strong>Total</strong></td> <td style="text-align:left;">#{'%02d:%02d:%02d' % fmt_time(total)}</td> </tr> </tfoot> </table> EOS output + tail elsif format == 'json' output = [] .reverse.each do |k, v| output << { 'tag' => k, 'seconds' => v, 'formatted' => '%02d:%02d:%02d' % fmt_time(v) } end output else output = [] .reverse.each do |k, v| spacer = '' (max - k.length).times do spacer += ' ' end output.push("#{k}:#{spacer}#{'%02d:%02d:%02d' % fmt_time(v)}") end output = output.empty? ? '' : "\n--- Tag Totals ---\n" + output.join("\n") output += "\n\nTotal tracked: #{'%02d:%02d:%02d' % fmt_time(total)}\n" output end end |
#today(times = true, output = nil, opt = {}) ⇒ Object
1805 1806 1807 1808 1809 1810 1811 1812 |
# File 'lib/doing/wwid.rb', line 1805 def today(times = true, output = nil, opt = {}) opt[:totals] ||= false opt[:sort_tags] ||= false cfg = @config['templates']['today'] list_section({ section: opt[:section], wrap_width: cfg['wrap_width'], count: 0, format: cfg['date_format'], template: cfg['template'], order: 'asc', today: true, times: times, output: output, totals: opt[:totals], sort_tags: opt[:sort_tags] }) end |
#update_item(old_item, new_item) ⇒ Object
991 992 993 994 995 996 997 998 999 1000 |
# File 'lib/doing/wwid.rb', line 991 def update_item(old_item, new_item) section = old_item['section'] section_items = @content[section]['items'] s_idx = section_items.index(old_item) section_items[s_idx] = new_item @results.push("Entry updated: #{section_items[s_idx]['title']}") @content[section]['items'] = section_items end |
#views ⇒ Array
Returns View names.
1245 1246 1247 |
# File 'lib/doing/wwid.rb', line 1245 def views @config.has_key?('views') ? @config['views'].keys : [] end |
#write(file = nil) ⇒ Object
1180 1181 1182 1183 1184 1185 1186 1187 1188 1189 1190 1191 1192 1193 1194 1195 1196 1197 1198 1199 1200 1201 1202 1203 1204 1205 1206 1207 1208 1209 |
# File 'lib/doing/wwid.rb', line 1180 def write(file = nil) output = @other_content_top ? "#{@other_content_top.join("\n")}\n" : '' @content.each do |title, section| output += "#{section['original']}\n" output += list_section({ section: title, template: "\t- %date | %title%note", highlight: false }) end output += @other_content_bottom.join("\n") unless @other_content_bottom.nil? if file.nil? $stdout.puts output else file = File.(file) if File.exist?(file) # Create a backup copy for the undo command FileUtils.cp(file, "#{file}~") File.open(file, 'w+') do |f| f.puts output end end if @config.key?('run_after') stdout, stderr, status = Open3.capture3(@config['run_after']) if status.exitstatus.positive? warn "Error running #{@config['run_after']}" warn stderr end end end end |
#yesterday(section, times = nil, output = nil, opt = {}) ⇒ Object
1842 1843 1844 1845 1846 1847 1848 |
# File 'lib/doing/wwid.rb', line 1842 def yesterday(section, times = nil, output = nil, opt = {}) opt[:totals] ||= false opt[:sort_tags] ||= false section = guess_section(section) list_section({ section: section, count: 0, order: 'asc', yesterday: true, times: times, output: output, totals: opt[:totals], sort_tags: opt[:sort_tags] }) end |
#yn(question, default_response: false) ⇒ Bool
Returns yes or no.
549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 578 579 580 581 582 583 584 |
# File 'lib/doing/wwid.rb', line 549 def yn(question, default_response: false) default = default_response ? 'y' : 'n' # if this isn't an interactive shell, answer default return default.downcase == 'y' unless $stdout.isatty # clear the buffer if ARGV&.length ARGV.length.times do ARGV.shift end end system 'stty cbreak' cw = colors['white'] cbw = colors['boldwhite'] cbg = colors['boldgreen'] cd = colors['default'] = if default default =~ /y/i ? "#{cw}[#{cbg}Y#{cw}/#{cbw}n#{cw}]#{cd}" : "#{cw}[#{cbw}y#{cw}/#{cbg}N#{cw}]#{cd}" else "#{cw}[#{cbw}y#{cw}/#{cbw}n#{cw}]#{cd}" end $stdout.syswrite "#{cbw}#{question.sub(/\?$/, '')} #{}#{cbw}?#{cd} " res = $stdin.sysread 1 puts system 'stty cooked' res.chomp! res.downcase! res = default.downcase if res == '' res =~ /y/i end |